# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json

from knack.log import get_logger
from knack.util import CLIError


logger = get_logger(__name__)


def duplicate_resource_exception_handler(ex):
    # wraps DocumentDB 409 error in CLIError
    from azure.cosmos.errors import HTTPFailure
    if isinstance(ex, HTTPFailure) and ex.status_code == 409:
        raise CLIError(
            'Operation Failed: Resource Already Exists')
    raise ex


def resource_not_found_exception_handler(ex):
    # wraps DocumentDB 404 error in CLIError
    from azure.cosmos.errors import HTTPFailure
    if isinstance(ex, HTTPFailure) and ex.status_code == 404:
        raise CLIError('Operation Failed: Resource Not Found')
    raise ex


def invalid_arg_found_exception_handler(ex):
    # wraps DocumentDB 400 error in CLIError
    from azure.cosmos.errors import HTTPFailure
    if isinstance(ex, HTTPFailure) and ex.status_code == 400:
        cli_error = None
        try:
            if ex._http_error_message:  # pylint:disable=protected-access
                msg = json.loads(ex._http_error_message)  # pylint:disable=protected-access
                if msg['message']:
                    msg = msg['message'].split('\n')[0]
                    msg = msg[len('Message: '):] if msg.find('Message: ') == 0 else msg
                    cli_error = CLIError('Operation Failed: Invalid Arg {}'.format(msg))
        except Exception:  # pylint:disable=broad-except
            pass

        if cli_error:
            raise cli_error  # pylint:disable=raising-bad-type

        raise CLIError('Operation Failed: Invalid Arg {}'.format(str(ex)))
    raise ex


def unknown_server_failure_exception_handler(ex):
    # wraps unknown documentdb error in CLIError
    from azure.cosmos.errors import HTTPFailure
    if isinstance(ex, HTTPFailure):
        raise CLIError('Operation Failed: {}'.format(str(ex)))
    raise ex


def exception_handler_chain_builder(handlers):
    # creates a handler which chains the handler
    # as soon as one of the chained handlers raises CLIError, it raises CLIError
    # if no handler raises CLIError it raises the original exception
    def chained_handler(ex):
        if isinstance(ex, CLIError):
            raise ex
        for h in handlers:
            try:
                h(ex)
            except CLIError as cli_error:
                raise cli_error
            except Exception:  # pylint:disable=broad-except
                pass
        raise ex

    return chained_handler


def network_exception_handler(ex):
    # wraps a connection exception in CLIError
    import requests.exceptions
    if isinstance(ex, (requests.exceptions.ConnectionError, requests.exceptions.HTTPError)):
        raise CLIError('Please ensure you have network connection. Error detail: ' + str(ex))
    raise ex


def generic_exception_handler(ex):
    logger.debug(ex)
    chained_handler = exception_handler_chain_builder([duplicate_resource_exception_handler,
                                                       resource_not_found_exception_handler,
                                                       invalid_arg_found_exception_handler,
                                                       unknown_server_failure_exception_handler,
                                                       network_exception_handler])
    chained_handler(ex)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


from azure.cli.core.util import CLIError, sdk_no_wait
from azure.mgmt.media.models import (AssetTrack, TextTrack)


def get_asset_track(client, account_name, resource_group_name, asset_name, track_name):
    return client.get(resource_group_name, account_name, asset_name, track_name)


def create_asset_track(client, account_name, resource_group_name, asset_name, track_name, track_type, file_name=None,
                       display_name=None, player_visibility=None, language_code=None, no_wait=False):
    track = None
    if track_type == 'Text':
        track = TextTrack(file_name=file_name, display_name=display_name, player_visibility=player_visibility,
                          language_code=language_code)
    else:
        raise CLIError("'{}' is not a valid track type. Currently supported track type is Text.".format(track_type))

    return sdk_no_wait(no_wait, client.begin_create_or_update, resource_group_name=resource_group_name,
                       account_name=account_name, asset_name=asset_name, track_name=track_name,
                       parameters=AssetTrack(track=track))


def update_track_data(client, account_name, resource_group_name, asset_name, track_name, no_wait=False):
    return sdk_no_wait(no_wait, client.begin_update_track_data, resource_group_name=resource_group_name,
                       account_name=account_name, asset_name=asset_name, track_name=track_name)


def update_track(client, account_name, resource_group_name, asset_name, track_name,
                 display_name=None, player_visibility=None, language_code=None, no_wait=False):
    current_track = client.get(resource_group_name, account_name, asset_name, track_name)

    if display_name:
        current_track.track.display_name = display_name
    if player_visibility:
        current_track.track.player_visibility = player_visibility
    if language_code:
        current_track.track.language_code = language_code

    return sdk_no_wait(no_wait, client.begin_update, resource_group_name=resource_group_name,
                       account_name=account_name, asset_name=asset_name, track_name=track_name,
                       parameters=current_track)



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['sql'] = """
type: group
short-summary: Manage Azure SQL Databases and Data Warehouses.
"""

helps['sql db'] = """
type: group
short-summary: Manage databases.
"""

helps['sql db audit-policy'] = """
type: group
short-summary: Manage a database's auditing policy.
"""

helps['sql db audit-policy wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the database's audit policy is met.
examples:
  - name: Place the CLI in a waiting state until it determines that database's audit policy exists
    text: az sql db audit-policy wait -g mygroup -s myserver -n mydb --exists
"""

helps['sql db audit-policy update'] = """
type: command
short-summary: Update a database's auditing policy.
long-summary: If the policy is being enabled, `--storage-account` or both `--storage-endpoint` and `--storage-key` must be specified.
examples:
  - name: Enable by storage account name.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb --state Enabled \\
            --bsts Enabled --storage-account mystorage
  - name: Enable by storage endpoint and key.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb --state Enabled \\
            --bsts Enabled --storage-endpoint https://mystorage.blob.core.windows.net \\
            --storage-key MYKEY==
  - name: Set the list of audit actions.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb \\
            --actions FAILED_DATABASE_AUTHENTICATION_GROUP 'UPDATE on database::mydb by public'
  - name: Disable an auditing policy.
    text: az sql db audit-policy update -g mygroup -s myserver -n mydb --state Disabled
  - name: Disable a blob storage auditing policy.
    text: az sql db audit-policy update -g mygroup -s myserver -n mydb --bsts Disabled
  - name: Enable a log analytics auditing policy.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb --state Enabled \\
            --lats Enabled --lawri myworkspaceresourceid
  - name: Disable a log analytics auditing policy.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb
            --lats Disabled
  - name: Enable an event hub auditing policy.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb --state Enabled \\
            --event-hub-target-state Enabled \\
            --event-hub-authorization-rule-id eventhubauthorizationruleid \\
            --event-hub eventhubname
  - name: Enable an event hub auditing policy for default event hub.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb --state Enabled \\
            --event-hub-target-state Enabled \\
            --event-hub-authorization-rule-id eventhubauthorizationruleid
  - name: Disable an event hub auditing policy.
    text: |
        az sql db audit-policy update -g mygroup -s myserver -n mydb
            --event-hub-target-state Disabled
"""

helps['sql db copy'] = """
type: command
short-summary: Create a copy of a database.
long-summary: A full list of performance level options can be seen by executing `az sql db list-editions -a -o table -l LOCATION`. The copy destination database must have the same edition as the source database, but you can change the edition after the copy has completed.
examples:
  - name: Create a database with performance level S0 as a copy of an existing Standard database.
    text: az sql db copy -g mygroup -s myserver -n originalDb --dest-name newDb --service-objective S0
  - name: Create a database with GeneralPurpose edition, Gen4 hardware, and 1 vcore as a copy of an existing GeneralPurpose database.
    text: az sql db copy -g mygroup -s myserver -n originalDb --dest-name newDb -f Gen4 -c 1
  - name: Create a database with local backup storage redundancy as a copy of an existing database
    text: az sql db copy -g mygroup -s myserver -n originalDb --dest-name newDb --backup-storage-redundancy Local

"""

helps['sql db create'] = """
type: command
short-summary: Create a database.
long-summary: A full list of performance level options can be seen by executing `az sql db list-editions -a -o table -l LOCATION`.
examples:
  - name: Create a Standard S0 database.
    text: az sql db create -g mygroup -s myserver -n mydb --service-objective S0
  - name: Create a database with GeneralPurpose edition, Gen4 hardware and 1 vcore
    text: az sql db create -g mygroup -s myserver -n mydb -e GeneralPurpose -f Gen4 -c 1
  - name: Create a database with zone redundancy enabled
    text: az sql db create -g mygroup -s myserver -n mydb -z
  - name: Create a database with zone redundancy explicitly disabled
    text: az sql db create -g mygroup -s myserver -n mydb -z false
  - name: Create a GeneralPurpose Gen5 2 vcore serverless database with auto pause delay of 120 minutes
    text: az sql db create -g mygroup -s myserver -n mydb -e GeneralPurpose -f Gen5 -c 2 --compute-model Serverless --auto-pause-delay 120
  - name: Create a Hyperscale Gen5 2 vcore database with 2 read replicas
    text: az sql db create -g mygroup -s myserver -n mydb -e Hyperscale -f Gen5 -c 2 --read-replicas 2
  - name: Create a GeneralPurpose database with locally redundant backup storage
    text: az sql db create -g mygroup -s myserver -n mydb -e GeneralPurpose --backup-storage-redundancy Local
"""

helps['sql db delete'] = """
type: command
short-summary: Delete a database.
examples:
  - name: Delete a database. (autogenerated)
    text: az sql db delete --name MyAzureSQLDatabase --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql db export'] = """
type: command
short-summary: Export a database to a bacpac.
examples:
  - name: Get an SAS key for use in export operation.
    text: |
        az storage blob generate-sas --account-name myAccountName -c myContainer -n myBacpac.bacpac \\
            --permissions w --expiry 2018-01-01T00:00:00Z
  - name: Export bacpac using an SAS key.
    text: |
        az sql db export -s myserver -n mydatabase -g mygroup -p password -u login \\
            --storage-key "?sr=b&sp=rw&se=2018-01-01T00%3A00%3A00Z&sig=mysignature&sv=2015-07-08" \\
            --storage-key-type SharedAccessKey \\
            --storage-uri https://myAccountName.blob.core.windows.net/myContainer/myBacpac.bacpac
  - name: Export bacpac using a storage account key.
    text: |
        az sql db export -s myserver -n mydatabase -g mygroup -p password -u login \\
            --storage-key MYKEY== --storage-key-type StorageAccessKey \\
            --storage-uri https://myAccountName.blob.core.windows.net/myContainer/myBacpac.bacpac
"""

helps['sql db import'] = """
type: command
short-summary: Imports a bacpac into an existing database.
examples:
  - name: Get an SAS key for use in import operation.
    text: |
        az storage blob generate-sas --account-name myAccountName -c myContainer -n myBacpac.bacpac \\
            --permissions r --expiry 2018-01-01T00:00:00Z
  - name: Import bacpac into an existing database using an SAS key.
    text: |
        az sql db import -s myserver -n mydatabase -g mygroup -p password -u login \\
            --storage-key "?sr=b&sp=rw&se=2018-01-01T00%3A00%3A00Z&sig=mysignature&sv=2015-07-08" \\
            --storage-key-type SharedAccessKey \\
            --storage-uri https://myAccountName.blob.core.windows.net/myContainer/myBacpac.bacpac
  - name: Import bacpac into an existing database using a storage account key.
    text: |
        az sql db import -s myserver -n mydatabase -g mygroup -p password -u login --storage-key MYKEY== \\
            --storage-key-type StorageAccessKey \\
            --storage-uri https://myAccountName.blob.core.windows.net/myContainer/myBacpac.bacpac
"""

helps['sql db list'] = """
type: command
short-summary: List databases on a server or elastic pool.
examples:
  - name: List databases on a server or elastic pool. (autogenerated)
    text: az sql db list --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql db list-editions'] = """
type: command
short-summary: Show database editions available for the currently active subscription.
long-summary: Includes available service objectives and storage limits. In order to reduce verbosity, settings to intentionally reduce storage limits are hidden by default.
examples:
  - name: Show all database editions in a location.
    text: az sql db list-editions -l westus -o table
  - name: Show all available database service objectives for Standard edition.
    text: az sql db list-editions -l westus --edition Standard -o table
  - name: Show available max database sizes for P1 service objective
    text: az sql db list-editions -l westus --service-objective P1 --show-details max-size
"""

helps['sql db str-policy'] = """
type: group
short-summary: Manage SQL database short term retention policy.
"""

helps['sql db str-policy set'] = """
type: command
short-summary: Update short term retention settings for a live database.
examples:
  - name: Set short term retention for a live database.
    text: az sql db str-policy set -g mygroup -s myserver -n mydb --retention-days retentionindays --diffbackup-hours diffbackuphours
"""

helps['sql db str-policy show'] = """
type: command
short-summary: Show the short term retention policy for a live database.
examples:
  - name: Show short term retention policy for a live database.
    text: az sql db str-policy show -g mygroup -s myserver -n mydb
"""

helps['sql db str-policy wait'] = """
type: command
short-summary: Place the CLI in a waiting state until the policy is set.
"""

helps['sql db ltr-policy'] = """
type: group
short-summary: Manage SQL database long term retention policy.
"""

helps['sql db ltr-policy set'] = """
type: command
short-summary: Update long term retention settings for a database.
examples:
  - name: Set long term retention for a database.
    text: az sql db ltr-policy set -g mygroup -s myserver -n mydb --weekly-retention "P1W" --monthly-retention "P6M" --yearly-retention "P1Y" --week-of-year 26
"""

helps['sql db ltr-policy show'] = """
type: command
short-summary: Show the long term retention policy for a database.
examples:
  - name: Show long term retention policy for a database.
    text: az sql db ltr-policy show -g mygroup -s myserver -n mydb
"""

helps['sql db ltr-backup'] = """
type: group
short-summary: Manage SQL database long term retention backups.
"""

helps['sql db ltr-backup show'] = """
type: command
short-summary: Get a long term retention backup for a database.
examples:
  - name: Show long term retention backup for a database.
    text: az sql db ltr-backup show -l southeastasia -s myserver -d mydb -n "3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000"
"""

helps['sql db ltr-backup list'] = """
type: command
short-summary: List the long term retention backups for a location, server or database.
examples:
  - name: List long term retention backups for a database.
    text: az sql db ltr-backup list -l southeastasia -s myserver -d mydb
  - name: List long term retention backups for a server (list only the latest LTR backups, which belong to live databases).
    text: az sql db ltr-backup list -l southeastasia -s myserver --database-state Live --only-latest-per-database True
  - name: List long term retention backups for a server (with resource group argument).
    text: az sql db ltr-backup list -l southeastasia -g mygroup -s myserver
  - name: List long term retention backups for a location (list only the latest LTR backups, which belong to live databases).
    text: az sql db ltr-backup list -l southeastasia --database-state Live --only-latest-per-database True
  - name: List long term retention backups for a location (with resource group argument).
    text: az sql db ltr-backup list -l southeastasia -g mygroup
"""

helps['sql db ltr-backup delete'] = """
type: command
short-summary: Delete a long term retention backup.
examples:
  - name: Delete long term retention backup for database.
    text: az sql db ltr-backup delete -l southeastasia -s myserver -d mydb -n "3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000"
"""

helps['sql db ltr-backup restore'] = """
type: command
short-summary: Restore a long term retention backup to a new database.
examples:
  - name: Restore LTR backup.
    text: |
        az sql db ltr-backup restore \\
        --dest-database targetdb --dest-server myserver --dest-resource-group mygroup \\
        --backup-id "/subscriptions/6caa113c-794c-42f8-ab9d-878d8aa104dc/resourceGroups/mygroup/providers/Microsoft.Sql/locations/southeastasia/longTermRetentionServers/myserver/longTermRetentionDatabases/sourcedb/longTermRetentionBackups/3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000"
"""

helps['sql db ltr-backup wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the database is met.
"""

helps['sql db op'] = """
type: group
short-summary: Manage operations on a database.
"""

helps['sql db op cancel'] = """
type: command
examples:
  - name: Cancel an operation.
    text: az sql db op cancel -g mygroup -s myserver -d mydb -n d2896mydb-2ba8-4c84-bac1-387c430cce40
"""

helps['sql mi op cancel'] = """
type: command
examples:
  - name: Cancel an operation.
    text: az sql mi op cancel -g mygroup --mi myManagedInstance -n d2896mydb-2ba8-4c84-bac1-387c430cce40
"""

helps['sql db rename'] = """
type: command
short-summary: Rename a database.
examples:
  - name: Rename a database. (autogenerated)
    text: az sql db rename --name MyAzureSQLDatabase --new-name MyNew --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql db replica'] = """
type: group
short-summary: Manage replication between databases.
"""

helps['sql db replica create'] = """
type: command
short-summary: Create a database as a readable secondary replica of an existing database.
long-summary: A full list of performance level options can be seen by executing `az sql db list-editions -a -o table -l LOCATION`. The secondary database must have the same edition as the primary database.
examples:
  - name: Create a database with performance level S0 as a secondary replica of an existing Standard database.
    text: az sql db replica create -g mygroup -s myserver -n originalDb --partner-server newDb --service-objective S0
  - name: Create a database with GeneralPurpose edition, Gen4 hardware, and 1 vcore as a secondary replica of an existing GeneralPurpose database
    text: az sql db replica create -g mygroup -s myserver -n originalDb --partner-server newDb -f Gen4 -c 1
  - name: Create a database with with zone redundant backup storage as a secondary replica of an existing database.
    text: az sql db replica create -g mygroup -s myserver -n originalDb --partner-server newDb --backup-storage-redundancy Zone
"""

helps['sql db replica delete-link'] = """
type: command
short-summary: Permanently stop data replication between two database replicas.
"""

helps['sql db replica list-links'] = """
type: command
short-summary: List the replicas of a database and their replication status.
examples:
  - name: List the replicas of a database and their replication status. (autogenerated)
    text: az sql db replica list-links --name MyAzureSQLDatabase --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql db replica set-primary'] = """
type: command
short-summary: Set the primary replica database by failing over from the current primary replica database.
examples:
  - name: Set the primary replica database by failing over from the current primary replica database. (autogenerated)
    text: az sql db replica set-primary --allow-data-loss --name MyDatabase --resource-group MyResourceGroup --server myserver --subscription MySubscription
    crafted: true
"""

helps['sql db restore'] = """
type: command
short-summary: Create a new database by restoring from a backup.
examples:
  - name: Create a new database by restoring from a backup. (autogenerated)
    text: |
        az sql db restore --dest-name MyDest --edition GeneralPurpose --name MyAzureSQLDatabase --resource-group MyResourceGroup --server myserver --subscription MySubscription --time "2018-05-20T05:34:22"
    crafted: true
  - name: Create a new database with geo-redundant backup storage by restoring from a backup. (autogenerated)
    text: |
        az sql db restore --dest-name MyDest --edition GeneralPurpose --name MyAzureSQLDatabase --resource-group MyResourceGroup --server myserver --subscription MySubscription --time "2018-05-20T05:34:22" --backup-storage-redundancy Geo
"""

helps['sql db show'] = """
type: command
short-summary: Get the details for a database.
examples:
  - name: Get the details for a database. (autogenerated)
    text: az sql db show --name MyAzureSQLDatabase --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql db show-connection-string'] = """
type: command
short-summary: Generates a connection string to a database.
examples:
  - name: Generate connection string for ado.net
    text: az sql db show-connection-string -s myserver -n mydb -c ado.net
"""

helps['sql db tde'] = """
type: group
short-summary: Manage a database's transparent data encryption.
"""

helps['sql db tde set'] = """
type: command
short-summary: Sets a database's transparent data encryption configuration.
examples:
  - name: Sets a database's transparent data encryption configuration. (autogenerated)
    text: az sql db tde set --database mydb --resource-group MyResourceGroup --server myserver --status Enabled
    crafted: true
"""

helps['sql db threat-policy'] = """
type: group
short-summary: Manage a database's threat detection policies.
"""

helps['sql db threat-policy update'] = """
type: command
short-summary: Update a database's threat detection policy.
long-summary: If the policy is being enabled, storage_account or both storage_endpoint and storage_account_access_key must be specified.
examples:
  - name: Enable by storage account name.
    text: |
        az sql db threat-policy update -g mygroup -s myserver -n mydb \\
            --state Enabled --storage-account mystorage
  - name: Enable by storage endpoint and key.
    text: |
        az sql db threat-policy update -g mygroup -s myserver -n mydb \\
            --state Enabled --storage-endpoint https://mystorage.blob.core.windows.net \\
            --storage-key MYKEY==
  - name: Disable a subset of alert types.
    text: |
        az sql db threat-policy update -g mygroup -s myserver -n mydb \\
            --disabled-alerts Sql_Injection_Vulnerability Access_Anomaly
  - name: Configure email recipients for a policy.
    text: |
        az sql db threat-policy update -g mygroup -s myserver -n mydb \\
            --email-addresses me@examlee.com you@example.com \\
            --email-account-admins Enabled
  - name: Disable a threat policy.
    text: az sql db threat-policy update -g mygroup -s myserver -n mydb --state Disabled
"""

helps['sql db update'] = """
type: command
short-summary: Update a database.
examples:
  - name: Update a database to Standard edition, S0 performance level (10 DTU) by specifying DTU capacity. Note that GeneralPurpose allows a wider range of max size than Standard edition.
    text: az sql db update -g mygroup -s myserver -n mydb --edition Standard --capacity 10 --max-size 250GB
  - name: Update a database to Standard edition, S1 performance level (20 DTU) by specifying performance level name. Note that GeneralPurpose allows a wider range of max size than Standard edition.
    text: az sql db update -g mygroup -s myserver -n mydb --edition Standard --service-objective S1 --max-size 250GB
  - name: Update a database to GeneralPurpose edition, 4 vcores with Gen5 hardware
    text: az sql db update -g mygroup -s myserver -n mydb --edition GeneralPurpose --capacity 4 --family Gen5
  - name: Update database with increased max size
    text: az sql db update -g mygroup -s myserver -n mydb --max-size 500GB
  - name: Update database with zone redundancy enabled
    text: az sql db update -g mygroup -s myserver -n mydb -z
  - name: Update database with zone redundancy explicitly disabled
    text: az sql db update -g mygroup -s myserver -n mydb -z false
  - name: Update database to serverless compute model
    text: az sql db update -g mygroup -s myserver -n mydb --edition GeneralPurpose --capacity 2 --family Gen5 --compute-model Serverless
  - name: Update database with locally redundant backup storage
    text: az sql db update -g mygroup -s myserver -n mydb --backup-storage-redundancy Local

"""

helps['sql dw'] = """
type: group
short-summary: Manage data warehouses.
"""

helps['sql dw create'] = """
type: command
short-summary: Create a data warehouse.
examples:
  - name: Create a data warehouse. (autogenerated)
    text: az sql dw create --name MyDataWarehouse --resource-group MyResourceGroup --server myserver --service-objective S0
    crafted: true
"""

helps['sql dw delete'] = """
type: command
short-summary: Delete a data warehouse.
examples:
  - name: Delete a data warehouse. (autogenerated)
    text: az sql dw delete --name MyDataWarehouse --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql dw list'] = """
type: command
short-summary: List data warehouses for a server.
examples:
  - name: List data warehouses for a server. (autogenerated)
    text: az sql dw list --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql dw show'] = """
type: command
short-summary: Get the details for a data warehouse.
examples:
  - name: Get the details for a data warehouse. (autogenerated)
    text: az sql dw show --name MyDataWarehouse --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql dw update'] = """
type: command
short-summary: Update a data warehouse.
examples:
  - name: Update a data warehouse. (autogenerated)
    text: az sql dw update --name MyDataWarehouse --resource-group MyResourceGroup --server myserver --service-objective S1
    crafted: true
"""

helps['sql elastic-pool'] = """
type: group
short-summary: Manage elastic pools.
"""

helps['sql elastic-pool create'] = """
type: command
short-summary: Create an elastic pool.
examples:
  - name: Create elastic pool with zone redundancy enabled
    text: az sql elastic-pool create -g mygroup -s myserver -n mypool -z
  - name: Create elastic pool with zone redundancy explicitly disabled
    text: az sql elastic-pool create -g mygroup -s myserver -n mypool -z false
  - name: Create a Standard 100 DTU elastic pool.
    text: az sql elastic-pool create -g mygroup -s myserver -n mydb -e Standard -c 100
  - name: Create an elastic pool with GeneralPurpose edition, Gen4 hardware and 1 vcore.
    text: az sql elastic-pool create -g mygroup -s myserver -n mydb -e GeneralPurpose -f Gen4 -c 1
  - name: Create an elastic pool with Hyperscale edition, Gen5 hardware, 4 vcore and 2 high availability replicas.
    text: az sql elastic-pool create -g mygroup -s myserver -n mydb -e Hyperscale -f Gen5 -c 4 --ha-replicas 2
"""

helps['sql elastic-pool list-editions'] = """
type: command
short-summary: List elastic pool editions available for the active subscription.
long-summary: Also includes available pool DTU settings, storage limits, and per database settings. In order to reduce verbosity, additional storage limits and per database settings are hidden by default.
examples:
  - name: Show all elastic pool editions and pool DTU limits in the West US region.
    text: az sql elastic-pool list-editions -l westus -o table
  - name: Show all pool DTU limits for Standard edition in the West US region.
    text: az sql elastic-pool list-editions -l westus --edition Standard -o table
  - name: Show available max sizes for elastic pools with at least 100 DTUs in the West US region.
    text: az sql elastic-pool list-editions -l westus --dtu 100 --show-details max-size -o table
  - name: Show available per database settings for Standard 100 DTU elastic pools in the West US region.
    text: az sql elastic-pool list-editions -l westus --edition Standard --dtu 100 -o table --show-details db-min-dtu db-max-dtu db-max-size
"""

helps['sql elastic-pool op'] = """
type: group
short-summary: Manage operations on an elastic pool.
"""

helps['sql elastic-pool op cancel'] = """
type: command
examples:
  - name: Cancel an operation.
    text: az sql elastic-pool op cancel -g mygroup -s myserver --elastic-pool myelasticpool -n d2896mydb-2ba8-4c84-bac1-387c430cce40
"""

helps['sql elastic-pool update'] = """
type: command
short-summary: Update an elastic pool.
examples:
  - name: Update elastic pool with zone redundancy enabled
    text: az sql elastic-pool update -g mygroup -s myserver -n mypool -z
  - name: Update elastic pool with zone redundancy explicitly disabled
    text: az sql elastic-pool update -g mygroup -s myserver -n mypool -z false
  - name: Update elastic pool with 2 high availability replicas
    text: az sql elastic-pool update -g mygroup -s myserver -n mypool --ha-replicas 2
"""

helps['sql failover-group'] = """
type: group
short-summary: Manage SQL Failover Groups.
"""

helps['sql failover-group create'] = """
type: command
short-summary: Creates a failover group.
examples:
  - name: Creates a failover group. (autogenerated)
    text: az sql failover-group create --name MyFailoverGroup --partner-server newDb --resource-group MyResourceGroup --server myserver --subscription MySubscription
    crafted: true
"""

helps['sql failover-group set-primary'] = """
type: command
short-summary: Set the primary of the failover group by failing over all databases from the current primary server.
examples:
  - name: Set the primary of the failover group by failing over all databases from the current primary server. (autogenerated)
    text: az sql failover-group set-primary --name MyFailoverGroup --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql failover-group update'] = """
type: command
short-summary: Updates the failover group.
"""

helps['sql instance-failover-group'] = """
type: group
short-summary: Manage SQL Instance Failover Groups.
"""

helps['sql instance-failover-group create'] = """
type: command
short-summary: Creates an instance failover group between two connected managed instances.
long-summary: If an outage occurs on the primary server, the grace period indicates that Azure SQL Managed Database will not initiate automatic failover before the grace period expires. Please note that failover operation with --allow-data-loss option might cause data loss due to the nature of asynchronous synchronization.
"""

helps['sql instance-failover-group set-primary'] = """
type: command
short-summary: Set the primary of the instance failover group by failing over all databases from the current primary managed instance.
"""

helps['sql instance-failover-group update'] = """
type: command
short-summary: Updates the instance failover group.
"""

helps['sql instance-pool'] = """
type: group
short-summary: Manage instance pools.
"""

helps['sql instance-pool create'] = """
type: command
short-summary: Create an instance pool.
examples:
  - name: Example to create an instance pool (include --no-wait in the end to get an asynchronous experience)
    text: az sql instance-pool create -g resource_group_name -n instance_pool_name -l location --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName} --license-type LicenseIncluded --capacity 8 -e GeneralPurpose -f Gen5 --no-wait
  - name: Example to create an instance pool with subnet name and vnet-name
    text: az sql instance-pool create --license-type LicenseIncluded -l northcentralus -n myinstancepool -c 8 -e GeneralPurpose -f Gen5 -g billingPools --subnet mysubnetname --vnet-name myvnetname
"""

helps['sql instance-pool delete'] = """
type: command
short-summary: Delete an instance pool.
examples:
  - name: Delete an instance pool
    text: az sql instance-pool delete -g mygroup -n myinstancepool --yes
"""

helps['sql instance-pool list'] = """
type: command
short-summary: List available instance pools.
examples:
  - name: List all instance pools in the current subscription.
    text: az sql instance-pool list
  - name: List all instance pools in a resource group.
    text: az sql instance-pool list -g mygroup
"""

helps['sql instance-pool show'] = """
type: command
short-summary: Get the details for an instance pool.
examples:
  - name: Get the details for an instance pool
    text: az sql instance-pool show -g mygroup -n myinstancepool
"""

helps['sql instance-pool update'] = """
type: command
short-summary: Update an instance pool.
examples:
  - name: Update an instance pool with new tags (make sure they are space separated if there are multiple tags)
    text: az sql instance-pool update -n myinstancepool -g mygroup --tags mykey1=myvalue1 mykey2=myvalue2
  - name: Clear the tags assigned to an instance pool
    text: az sql instance-pool update -n myinstancepool -g mygroup --tags ""
"""

helps['sql instance-pool wait'] = """
type: command
short-summary: Wait for an instance pool to reach a desired state.
examples:
  - name: Wait until an instance pool gets created.
    text: az sql instance-pool wait -n myinstancepool -g mygroup --created
"""

helps['sql stg'] = """
type: group
short-summary: Manage Server Trust Groups.
"""

helps['sql stg create'] = """
type: command
short-summary: Create a Server Trust Group.
examples:
  - name: Create a Server Trust Group with specified resource ids of its members.
    text: az sql stg create -g resourcegroup -l location -n stg-name --trust-scope GlobalTransactions -m $mi1-id $mi2-id
"""

helps['sql stg show'] = """
type: command
short-summary: Retrieve a Server Trust Group.
examples:
  - name: Retrieve a Server Trust Group.
    text: az sql stg show -g resourcegroup -l location -n stg-name
"""

helps['sql stg delete'] = """
type: command
short-summary: Delete a Server Trust Group.
examples:
  - name: Delete a Server Trust Group.
    text: az sql stg delete -g resourcegroup -l location -n stg-name
"""

helps['sql stg list'] = """
type: command
short-summary: Retrieve a list of Server Trust Groups.
examples:
  - name: Retrieve a list of Server Trust Groups by instance.
    text: az sql stg list -g resourcegroup --instance-name mi1-name
  - name: Retrieve a list of Server Trust Groups by location.
    text: az sql stg list -g resourcegroup -l location
"""

helps['sql mi'] = """
type: group
short-summary: Manage SQL managed instances.
"""

helps['sql mi ad-admin'] = """
type: group
short-summary: Manage a managed instance's Active Directory administrator.
"""

helps['sql mi ad-admin create'] = """
type: command
short-summary: Creates a new managed instance Active Directory administrator.
"""

helps['sql mi ad-admin delete'] = """
type: command
short-summary: Deletes an existing managed instance Active Directory Administrator.
"""

helps['sql mi ad-admin list'] = """
type: command
short-summary: Returns a list of managed instance Active Directory Administrators.
"""

helps['sql mi ad-admin update'] = """
type: command
short-summary: Updates an existing managed instance Active Directory administrator.
"""

helps['sql mi ad-only-auth'] = """
type: group
short-summary: Manage a Managed Instance's Azure Active Directly only settings.
"""

helps['sql mi ad-only-auth enable'] = """
type: command
short-summary: Enable Azure Active Directly only Authentication for this Managed Instance.
examples:
  - name: Enable Active Directory only authentication for a managed instance
    text: az sql mi ad-only-auth enable --resource-group mygroup --name myMI
"""

helps['sql mi ad-only-auth disable'] = """
type: command
short-summary: Disable Azure Active Directly only Authentication for this Managed Instance.
examples:
  - name: Disable Active Directory only authentication for a managed instance
    text: az sql mi ad-only-auth disable --resource-group mygroup --name myMI
"""

helps['sql mi ad-only-auth get'] = """
type: command
short-summary: Get a specific Azure Active Directly only Authentication property.
examples:
  - name: Get Active Directory only authentication status for a managed instance
    text: az sql mi ad-only-auth get --resource-group mygroup --name myMI
"""

helps['sql mi create'] = """
type: command
short-summary: Create a managed instance.
examples:
  - name: Create a managed instance with minimal set of parameters
    text: az sql mi create -g mygroup -n myinstance -l mylocation -i -u myusername -p mypassword --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName}
  - name: Create a managed instance with specified parameters and with identity
    text: az sql mi create -g mygroup -n myinstance -l mylocation -i -u myusername -p mypassword --license-type LicenseIncluded --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName} --capacity 8 --storage 32GB --edition GeneralPurpose --family Gen5
  - name: Create managed instance with specified parameters and tags
    text: az sql mi create -g mygroup -n myinstance -l mylocation -i -u myusername -p mypassword --license-type LicenseIncluded --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName} --capacity 8 --storage 32GB --edition GeneralPurpose --family Gen5 --tags tagName1=tagValue1 tagName2=tagValue2
  - name: Create managed instance with specified parameters and backup storage redundancy specified
    text: az sql mi create -g mygroup -n myinstance -l mylocation -i -u myusername -p mypassword --license-type LicenseIncluded --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName} --capacity 8 --storage 32GB --edition GeneralPurpose --family Gen5 --backup-storage-redundancy Local
  - name: Create a managed instance with maintenance configuration
    text: az sql mi create -g mygroup -n myinstance -l mylocation -i -u myusername -p mypassword --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName} -m SQL_{Region}_{MaintenanceConfigName}
  - name: Create a managed instance with Service Principal enabled
    text: az sql mi create -g mygroup -n myinstance -l mylocation -i -u myusername -p mypassword --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName} --service-principal-type SystemAssigned
  - name: Create a managed instance without SQL Admin, with AAD admin and AD Only enabled
    text: az sql mi create --enable-ad-only-auth --external-admin-principal-type User --external-admin-name myUserName --external-admin-sid c5e964e2-6bb2-2222-1111-3b16ec0e1234 -g myResourceGroup -n miName --subnet /subscriptions/78975f9f-2222-1111-1111-29c42ac70000/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/vnet-test/subnets/ManagedInstance
  - name: Create a managed instance without SQL Admin, with AD admin, AD Only enabled, User ManagedIdenties and Identity Type is SystemAssigned,UserAssigned.
    text: az sql mi create --enable-ad-only-auth --external-admin-principal-type User --external-admin-name myUserName \\
              --external-admin-sid c5e964e2-6bb2-1111-1111-3b16ec0e1234 -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type SystemAssigned,UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --subnet /subscriptions/78975f9f-2222-1111-1111-29c42ac70000/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/vnet-test/subnets/ManagedInstance
  - name: Create a managed instance without SQL Admin, with AD admin, AD Only enabled, User ManagedIdenties and Identity Type is UserAssigned.
    text: az sql mi create --enable-ad-only-auth --external-admin-principal-type User --external-admin-name myUserName \\
              --external-admin-sid c5e964e2-6bb2-1111-1111-3b16ec0e1234 -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --subnet /subscriptions/78975f9f-2222-1111-1111-29c42ac70000/resourceGroups/myResourceGroup/providers/Microsoft.Network/virtualNetworks/vnet-test/subnets/ManagedInstance
"""

helps['sql mi delete'] = """
type: command
short-summary: Delete a managed instance.
examples:
  - name: Delete a managed instance
    text: az sql mi delete -g mygroup -n myinstance --yes
"""

helps['sql mi failover'] = """
type: command
short-summary: Failover a managed instance.
examples:
  - name: Failover a managed instance primary replica
    text: az sql mi failover -g mygroup -n myinstance
  - name: Failover a managed instance readable secodary replica
    text: az sql mi failover -g mygroup -n myinstance --replica-type ReadableSecondary
"""

helps['sql mi key'] = """
type: group
short-summary: Manage a SQL Instance's keys.
"""

helps['sql mi key create'] = """
type: command
short-summary: Creates a SQL Instance key.
"""

helps['sql mi key delete'] = """
type: command
short-summary: Deletes a SQL Instance key.
"""

helps['sql mi key show'] = """
type: command
short-summary: Shows a SQL Instance key.
"""

helps['sql mi list'] = """
type: command
short-summary: List available managed instances.
examples:
  - name: List all managed instances in the current subscription.
    text: az sql mi list
  - name: List all managed instances in a resource group.
    text: az sql mi list -g mygroup
"""

helps['sql mi op'] = """
type: group
short-summary: Manage operations on a managed instance.
"""

helps['sql mi show'] = """
type: command
short-summary: Get the details for a managed instance.
examples:
  - name: Get the details for a managed instance
    text: az sql mi show -g mygroup -n myinstance
"""

helps['sql mi tde-key'] = """
type: group
short-summary: Manage a SQL Instance's encryption protector.
"""

helps['sql mi tde-key set'] = """
type: command
short-summary: Sets the SQL Instance's encryption protector.
"""

helps['sql mi update'] = """
type: command
short-summary: Update a managed instance.
examples:
  - name: Updates a mi with specified parameters and with identity
    text: az sql mi update -g mygroup -n myinstance -i -p mypassword --license-type mylicensetype --capacity vcorecapacity --storage storagesize
  - name: Update mi edition and hardware family
    text: az sql mi update -g mygroup -n myinstance --tier GeneralPurpose --family Gen5
  - name: Add or update a tag.
    text: az sql mi update -g mygroup -n myinstance --set tags.tagName=tagValue
  - name: Remove a tag.
    text: az sql mi update -g mygroup -n myinstance --remove tags.tagName
  - name: Update a managed instance. (autogenerated)
    text: az sql mi update --name myinstance --proxy-override Default --resource-group mygroup --subscription MySubscription
    crafted: true
  - name: Update a managed instance. (autogenerated)
    text: az sql mi update --name myinstance --public-data-endpoint-enabled true --resource-group mygroup --subscription MySubscription
    crafted: true
  - name: Update a managed instance with maintenance configuration
    text: az sql mi update -g mygroup -n myinstance -m SQL_{Region}_{MaintenanceConfigName}
  - name: Remove maintenance configuration from managed instance
    text: az sql mi update -g mygroup -n myinstance -m SQL_Default
  - name: Update a managed instance with Service Principal
    text: az sql mi update -g mygroup -n myinstance --service-principal-type SystemAssigned
  - name: Update a managed instance with User Managed Identies and Identity Type is SystemAssigned,UserAssigned.
    text: az sql mi update -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type SystemAssigned,UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi
  - name: Update a managed instance with User Managed Identies and Identity Type is UserAssigned
    text: az sql mi update -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi
  - name: Move managed instance to another subnet
    text: az sql mi update -g myResourceGroup -n myServer -i \\
              --subnet /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
  - name: Update mi backup storage redundancy
    text: az sql mi update -g mygroup -n myinstance --bsr Local
"""

helps['sql midb'] = """
type: group
short-summary: Manage SQL managed instance databases.
"""

helps['sql midb create'] = """
type: command
short-summary: Create a managed database.
examples:
  - name: Create a managed database with specified collation
    text: az sql midb create -g mygroup --mi myinstance -n mymanageddb --collation Latin1_General_100_CS_AS_SC
"""

helps['sql midb update'] = """
type: command
short-summary: Update a managed database.
examples:
  - name: Update a managed database with specified tags
    text: az sql midb update -g mygroup --mi myinstance -n mymanageddb --tags tag1="value1"
"""

helps['sql midb delete'] = """
type: command
short-summary: Delete a managed database.
examples:
  - name: Delete a managed database
    text: az sql midb delete -g mygroup --mi myinstance -n mymanageddb --yes
"""

helps['sql midb list'] = """
type: command
short-summary: List managed databases on a managed instance.
examples:
  - name: List managed databases on a managed instance
    text: az sql midb list -g mygroup --mi myinstance
"""

helps['sql midb list-deleted'] = """
type: command
short-summary: List restorable deleted managed databases.
examples:
  - name: List all restorable deleted managed databases on Managed Instance .
    text: az sql midb list-deleted -g mygroup --mi myinstance
"""

helps['sql midb restore'] = """
type: command
short-summary: Restore a managed database.
examples:
  - name: Restore a live managed database using Point in time restore
    text: az sql midb restore -g mygroup --mi myinstance -n mymanageddb --dest-name targetmidb --time "2018-05-20T05:34:22"
  - name: Restore a dropped managed database using Point in time restore
    text: az sql midb restore -g mygroup --mi myinstance -n mymanageddb --dest-name targetmidb --time "2018-05-20T05:34:22" --deleted-time "2018-05-20T05:34:22"
  - name: Restore a live managed database from another instance using Point in time restore
    text: az sql midb restore -g mygroup --mi myinstance -n mymanageddb --dest-name targetmidb --time "2018-05-20T05:34:22" --dest-mi targetmi --dest-resource-group targetrg
"""

helps['sql midb show'] = """
type: command
short-summary: Get the details for a managed database.
examples:
  - name: Get the details for a managed database
    text: az sql midb show -g mygroup --mi myinstance -n mymanageddb
"""

helps['sql midb short-term-retention-policy'] = """
type: group
short-summary: Manage SQL Managed Instance database backup short term retention policy.
"""

helps['sql midb short-term-retention-policy set'] = """
type: command
short-summary: Update short term retention for automated backups on a single database.
examples:
  - name: Set backup short term retention for live managed database.
    text: az sql midb short-term-retention-policy set -g mygroup --mi myinstance -n mymanageddb --retention-days retentionindays
  - name: Set backup short term retention for dropped managed database.
    text: az sql midb short-term-retention-policy set -g mygroup --mi myinstance -n mymanageddb --deleted-time "2018-05-20T05:34:22" --retention-days retentionindays
"""

helps['sql midb short-term-retention-policy show'] = """
type: command
short-summary: Show short term retention for automated backups on a single database.
examples:
  - name: Shows backup short term retention for live managed database.
    text: az sql midb short-term-retention-policy show -g mygroup --mi myinstance -n mymanageddb
  - name: Show backup short term retention for dropped managed database.
    text: az sql midb short-term-retention-policy show -g mygroup --mi myinstance -n mymanageddb --deleted-time "2018-05-20T05:34:22"
"""

helps['sql midb ltr-policy'] = """
type: group
short-summary: Manage SQL Managed Instance database long term retention policy.
"""

helps['sql midb ltr-policy set'] = """
type: command
short-summary: Update long term retention settings for a managed database.
examples:
  - name: Set long term retention for a managed database.
    text: az sql midb ltr-policy set -g mygroup --mi myinstance -n mymanageddb --weekly-retention "P1W" --monthly-retention "P6M" --yearly-retention "P1Y" --week-of-year 26
"""

helps['sql midb ltr-policy show'] = """
type: command
short-summary: Show the long term retention policy for a managed database.
examples:
  - name: Show long term retention policy for a managed database.
    text: az sql midb ltr-policy show -g mygroup --mi myinstance -n mymanageddb
"""

helps['sql midb ltr-backup'] = """
type: group
short-summary: Manage SQL Managed Instance database long term retention backups.
"""

helps['sql midb ltr-backup show'] = """
type: command
short-summary: Get a long term retention backup for a managed database.
examples:
  - name: Show long term retention backup for a managed database.
    text: az sql midb ltr-backup show -l southeastasia --mi myinstance -d mymanageddb -n "3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000"
    name: Show long term retention backup for a managed database.
    text: az sql midb ltr-backup show --backup-id '/subscriptions/6caa113c-794c-42f8-ab9d-878d8aa104dc/resourceGroups/mygroup/providers/Microsoft.Sql/locations/southeastasia/longTermRetentionManagedInstances/myinstance/longTermRetentionDatabases/mymanageddb/longTermRetentionManagedInstanceBackups/3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000'
"""

helps['sql midb ltr-backup list'] = """
type: command
short-summary: List the long term retention backups for a location, instance or database.
examples:
  - name: List long term retention backups for a managed database.
    text: az sql midb ltr-backup list -l southeastasia --mi myinstance -d mymanageddb
  - name: List long term retention backups for a managed instance (list only the latest LTR backups, which belong to live databases).
    text: az sql midb ltr-backup list -l southeastasia --mi myinstance --database-state Live --only-latest-per-database
  - name: List long term retention backups for a managed instance (with resource group argument).
    text: az sql midb ltr-backup list -l southeastasia -g mygroup --mi myinstance
  - name: List long term retention backups for a location (list only the latest LTR backups, which belong to live databases).
    text: az sql midb ltr-backup list -l southeastasia --database-state Live --only-latest-per-database
  - name: List long term retention backups for a location (with resource group argument).
    text: az sql midb ltr-backup list -l southeastasia -g mygroup
"""

helps['sql midb ltr-backup delete'] = """
type: command
short-summary: Delete a long term retention backup.
examples:
  - name: Delete long term retention backup for a managed database.
    text: az sql midb ltr-backup delete -l southeastasia --mi myinstance -d mymanageddb --name "3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000"
  - name: Delete long term retention backup for a managed database.
    text: az sql midb ltr-backup delete --backup-id '/subscriptions/6caa113c-794c-42f8-ab9d-878d8aa104dc/resourceGroups/mygroup/providers/Microsoft.Sql/locations/southeastasia/longTermRetentionManagedInstances/myinstance/longTermRetentionDatabases/mymanageddb/longTermRetentionManagedInstanceBackups/3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000'
"""

helps['sql midb ltr-backup restore'] = """
type: command
short-summary: Restore a long term retention backup to a new database.
examples:
  - name: Restore a managed database using LTR backup.
    text: |
            az sql midb ltr-backup restore \\
                --dest-database targetmidb --dest-mi myinstance --dest-resource-group mygroup \\
                --backup-id "/subscriptions/6caa113c-794c-42f8-ab9d-878d8aa104dc/resourceGroups/mygroup/providers/Microsoft.Sql/locations/southeastasia/longTermRetentionManagedInstances/myinstance/longTermRetentionDatabases/sourcemidb/longTermRetentionManagedInstanceBackups/3214b3fb-fba9-43e7-96a3-09e35ffcb336;132292152080000000"
"""

helps['sql midb ltr-backup wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the managed database is met.
"""

helps['sql midb log-replay'] = """
type: group
short-summary: SQL Managed Instance database Log Replay service commands.
"""

helps['sql midb log-replay start'] = """
type: command
short-summary: Start Log Replay service on specified database.
examples:
  - name: Start Log Replay service with auto complete option.
    text: az sql midb log-replay start -g mygroup --mi myinstance -n mymanageddb -a --last-bn "backup.bak" --storage-uri "https://test.blob.core.windows.net/testing" --storage-sas "sv=2019-02-02&ss=b&srt=sco&sp=rl&se=2023-12-02T00:09:14Z&st=2019-11-25T16:09:14Z&spr=https&sig=92kAe4QYmXaht%2Fgjocqwerqwer41s%3D"
  - name: Start Log Replay service without auto complete option.
    text: az sql midb log-replay start -g mygroup --mi myinstance -n mymanageddb --storage-uri "https://test.blob.core.windows.net/testing" --storage-sas "sv=2019-02-02&ss=b&srt=sco&sp=rl&se=2023-12-02T00:09:14Z&st=2019-11-25T16:09:14Z&spr=https&sig=92kAe4QYmXaht%2Fgjocqwerqwer41s%3D"
"""

helps['sql midb log-replay complete'] = """
type: command
short-summary: Complete Log Replay service on specified database.
examples:
  - name: Complete log replay service.
    text: az sql midb log-replay complete -g mygroup --mi myinstance -n mymanageddb --last-backup-name "backup.bak"
"""

helps['sql midb log-replay show'] = """
type: command
short-summary: Get status of Log Replay service.
examples:
  - name: Get status of the ongoing log replay service.
    text: az sql midb log-replay show -g mygroup --mi myinstance -n mymanageddb
"""

helps['sql midb log-replay stop'] = """
type: command
short-summary: Stop Log Replay service.
examples:
  - name: Stop ongoing log replay service by deleting database.
    text: az sql midb log-replay stop -g mygroup --mi myinstance -n mymanageddb
"""

helps['sql midb log-replay wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the managed database is met.
examples:
  - name: Place the CLI in a waiting state until a condition of the managed database is met after starting log replay service.
    text: az sql midb log-replay wait --exists --resource-group mygroup --managed-instance myinstance --name mymanageddb
  - name: Place the CLI in a waiting state until a condition of the managed database is met after stopping log replay service.
    text: az sql midb log-replay wait --deleted --resource-group mygroup --managed-instance myinstance --name mymanageddb
"""

helps['sql server'] = """
type: group
short-summary: Manage SQL servers.
"""

helps['sql server ad-admin'] = """
type: group
short-summary: Manage a server's Active Directory administrator.
"""

helps['sql server ad-admin create'] = """
type: command
short-summary: Create a new server Active Directory administrator.
"""

helps['sql server ad-admin update'] = """
type: command
short-summary: Update an existing server Active Directory administrator.
"""

helps['sql server ad-only-auth'] = """
type: group
short-summary: Manage Azure Active Directly only Authentication settings for this Server.
"""

helps['sql server ad-only-auth enable'] = """
type: command
short-summary: Enable Azure Active Directly only Authentication for this Server.
examples:
  - name: Enable Active Directory only authentication for a sql server
    text: az sql server ad-only-auth enable --resource-group mygroup --name myServer
"""

helps['sql server ad-only-auth disable'] = """
type: command
short-summary: Disable Azure Active Directly only Authentication for this Server.
examples:
  - name: Disable Active Directory only authentication for a sql server
    text: az sql server ad-only-auth disable --resource-group mygroup --name myServer
"""

helps['sql server ad-only-auth get'] = """
type: command
short-summary: Get a specific Azure Active Directly only Authentication property.
examples:
  - name: Get Active Directory only authentication status for a sql server
    text: az sql server ad-only-auth get --resource-group mygroup --name myServer
"""

helps['sql server audit-policy'] = """
type: group
short-summary: Manage a server's auditing policy.
"""

helps['sql server audit-policy wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the server's audit policy is met.
examples:
  - name: Place the CLI in a waiting state until it determines that server's audit policy exists
    text: az sql server audit-policy wait -g mygroup -n myserver --exists
"""

helps['sql server audit-policy update'] = """
type: command
short-summary: Update a server's auditing policy.
long-summary: If the policy is being enabled, `--storage-account` or both `--storage-endpoint` and `--storage-key` must be specified.
examples:
  - name: Enable by storage account name.
    text: |
        az sql server audit-policy update -g mygroup -n myserver --state Enabled \\
            --bsts Enabled --storage-account mystorage
  - name: Enable by storage endpoint and key.
    text: |
        az sql server audit-policy update -g mygroup -n myserver --state Enabled \\
            --bsts Enabled --storage-endpoint https://mystorage.blob.core.windows.net \\
            --storage-key MYKEY==
  - name: Set the list of audit actions.
    text: |
        az sql server audit-policy update -g mygroup -n myserver \\
            --actions FAILED_DATABASE_AUTHENTICATION_GROUP 'UPDATE on server::myserver by public'
  - name: Disable an auditing policy.
    text: az sql server audit-policy update -g mygroup -n myserver --state Disabled
  - name: Disable a blob storage auditing policy.
    text: az sql server audit-policy update -g mygroup -n myserver --bsts Disabled
  - name: Enable a log analytics auditing policy.
    text: |
        az sql server audit-policy update -g mygroup -n myserver --state Enabled \\
            --lats Enabled --lawri myworkspaceresourceid
  - name: Disable a log analytics auditing policy.
    text: |
        az sql server audit-policy update -g mygroup -n myserver
            --lats Disabled
  - name: Enable an event hub auditing policy.
    text: |
        az sql server audit-policy update -g mygroup -n myserver --state Enabled \\
            --event-hub-target-state Enabled \\
            --event-hub-authorization-rule-id eventhubauthorizationruleid \\
            --event-hub eventhubname
  - name: Enable an event hub auditing policy for default event hub.
    text: |
        az sql server audit-policy update -g mygroup -n myserver --state Enabled \\
            --event-hub-target-state Enabled \\
            --event-hub-authorization-rule-id eventhubauthorizationruleid
  - name: Disable an event hub auditing policy.
    text: |
        az sql server audit-policy update -g mygroup -n myserver
            --event-hub-target-state Disabled
"""

helps['sql server ms-support'] = """
type: group
short-summary: Manage a server's Microsoft support operations.
"""

helps['sql server ms-support audit-policy'] = """
type: group
short-summary: Manage a server's Microsoft support operations auditing policy.
"""

helps['sql server ms-support audit-policy wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the server's Microsoft support operations audit policy is met.
examples:
  - name: Place the CLI in a waiting state until it determines that server's Microsoft support operations audit policy exists
    text: az sql server ms-support audit-policy wait -g mygroup -n myserver --exists
"""

helps['sql server ms-support audit-policy update'] = """
type: command
short-summary: Update a server's Microsoft support operations auditing policy.
long-summary: If the Microsoft support operations policy is being enabled, `--storage-account` or both `--storage-endpoint` and `--storage-key` must be specified.
examples:
  - name: Enable by storage account name.
    text: |
        az sql server ms-support audit-policy update -g mygroup -n myserver --state Enabled \\
            --bsts Enabled --storage-account mystorage
  - name: Enable by storage endpoint and key.
    text: |
        az sql server ms-support audit-policy update -g mygroup -n myserver --state Enabled \\
            --bsts Enabled --storage-endpoint https://mystorage.blob.core.windows.net \\
            --storage-key MYKEY==
  - name: Disable a Microsoft support operations auditing policy.
    text: az sql server ms-support audit-policy update -g mygroup -n myserver --state Disabled
  - name: Disable a blob storage Microsoft support operations auditing policy.
    text: az sql server ms-support audit-policy update -g mygroup -n myserver --bsts Disabled
  - name: Enable a log analytics Microsoft support operations auditing policy.
    text: |
        az sql server ms-support audit-policy update -g mygroup -n myserver --state Enabled \\
            --lats Enabled --lawri myworkspaceresourceid
  - name: Disable a log analytics Microsoft support operations auditing policy.
    text: |
        az sql server ms-support audit-policy update -g mygroup -n myserver
            --lats Disabled
  - name: Enable an event hub Microsoft support operations auditing policy.
    text: |
        az sql server ms-support audit-policy update -g mygroup -n myserver --state Enabled \\
            --event-hub-target-state Enabled \\
            --event-hub-authorization-rule-id eventhubauthorizationruleid \\
            --event-hub eventhubname
  - name: Enable an event hub Microsoft support operations auditing policy for default event hub.
    text: |
        az sql server ms-support audit-policy update -g mygroup -n myserver --state Enabled \\
            --event-hub-target-state Enabled \\
            --event-hub-authorization-rule-id eventhubauthorizationruleid
  - name: Disable an event hub Microsoft support operations auditing policy.
    text: |
        az sql server ms-support audit-policy update -g mygroup -n myserver
            --event-hub-target-state Disabled
"""

helps['sql server conn-policy'] = """
type: group
short-summary: Manage a server's connection policy.
"""

helps['sql server conn-policy show'] = """
type: command
short-summary: Gets a server's secure connection policy.
examples:
  - name: Gets a server's secure connection policy. (autogenerated)
    text: az sql server conn-policy show --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql server conn-policy update'] = """
type: command
short-summary: Updates a server's secure connection policy.
examples:
  - name: Updates a server's secure connection policy. (autogenerated)
    text: az sql server conn-policy update --connection-type Default --resource-group MyResourceGroup --server myserver
    crafted: true
"""

helps['sql server create'] = """
type: command
short-summary: Create a server.
examples:
  - name: Create a server.
    text: az sql server create -l westus -g mygroup -n myserver -u myadminuser -p myadminpassword
  - name: Create a server with disabled public network access to server.
    text: az sql server create -l westus -g mygroup -n myserver -u myadminuser -p myadminpassword -e false
  - name: Create a server without SQL Admin, with AD admin and AD Only enabled.
    text: az sql server create --enable-ad-only-auth --external-admin-principal-type User --external-admin-name myUserName --external-admin-sid c5e964e2-6bb2-1111-1111-3b16ec0e1234 -g myResourceGroup -n myServer
  - name: Create a server without SQL Admin, with AD admin, AD Only enabled, User ManagedIdenties and Identity Type is SystemAssigned,UserAssigned.
    text: az sql server create --enable-ad-only-auth --external-admin-principal-type User --external-admin-name myUserName \\
              --external-admin-sid c5e964e2-6bb2-1111-1111-3b16ec0e1234 -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type SystemAssigned,UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi
  - name: Create a server without SQL Admin, with AD admin, AD Only enabled, User ManagedIdenties and Identity Type is UserAssigned.
    text: az sql server create --enable-ad-only-auth --external-admin-principal-type User --external-admin-name myUserName \\
              --external-admin-sid c5e964e2-6bb2-1111-1111-3b16ec0e1234 -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi
"""

helps['sql server dns-alias'] = """
type: group
short-summary: Manage a server's DNS aliases.
"""

helps['sql server dns-alias set'] = """
type: command
short-summary: Sets a server to which DNS alias should point
"""

helps['sql server firewall-rule'] = """
type: group
short-summary: Manage a server's firewall rules.
"""

helps['sql server firewall-rule create'] = """
type: command
short-summary: Create a firewall rule.
examples:
  - name: Create a firewall rule
    text: az sql server firewall-rule create -g mygroup -s myserver -n myrule --start-ip-address 1.2.3.4 --end-ip-address 5.6.7.8
  - name: Create a firewall rule that allows access from Azure services
    text: az sql server firewall-rule create -g mygroup -s myserver -n myrule --start-ip-address 0.0.0.0 --end-ip-address 0.0.0.0
"""

helps['sql server firewall-rule list'] = """
type: command
short-summary: List a server's firewall rules.
examples:
  - name: List a server's firewall rules
    text: az sql server firewall-rule list -g mygroup -s myserver
"""

helps['sql server firewall-rule show'] = """
type: command
short-summary: Shows the details for a firewall rule.
examples:
  - name: Show a firewall rule
    text: az sql server firewall-rule show -g mygroup -s myserver -n myrule
"""

helps['sql server firewall-rule update'] = """
type: command
short-summary: Update a firewall rule.
examples:
  - name: Update a firewall rule
    text: az sql server firewall-rule update -g mygroup -s myserver -n myrule --start-ip-address 5.4.3.2 --end-ip-address 9.8.7.6
"""

helps['sql server outbound-firewall-rule'] = """
type: group
short-summary: Manage a server's outbound firewall rules.
"""

helps['sql server outbound-firewall-rule create'] = """
type: command
short-summary: Create a new outbound firewall rule.
examples:
  - name: Create a new outbound firewall rule
    text: az sql server outbound-firewall-rule create -g mygroup -s myserver -n allowedFQDN
  - name: Create a new outbound firewall rule
    text: az sql server outbound-firewall-rule create -g mygroup -s myserver --outbound-rule-fqdn allowedFQDN
"""

helps['sql server outbound-firewall-rule list'] = """
type: command
short-summary: List a server's outbound firewall rules.
examples:
  - name: List a server's outbound firewall rules
    text: az sql server outbound-firewall-rule list -g mygroup -s myserver
"""

helps['sql server outbound-firewall-rule show'] = """
type: command
short-summary: Show the details for an outbound firewall rule.
examples:
  - name: Show the outbound firewall rule
    text: az sql server outbound-firewall-rule show -g mygroup -s myserver -n myrule
  - name: Show the outbound firewall rule
    text: az sql server outbound-firewall-rule show -g mygroup -s myserver --outbound-rule-fqdn allowedFQDN
"""

helps['sql server outbound-firewall-rule delete'] = """
type: command
short-summary: Delete the outbound firewall rule.
examples:
  - name: Delete the outbound firewall rule
    text: az sql server outbound-firewall-rule delete -g mygroup -s myserver -n myrule
  - name: Delete the outbound firewall rule
    text: az sql server outbound-firewall-rule delete -g mygroup -s myserver --outbound-rule-fqdn allowedFQDN
"""

helps['sql server key'] = """
type: group
short-summary: Manage a server's keys.
"""

helps['sql server key create'] = """
type: command
short-summary: Creates a server key.
"""

helps['sql server key delete'] = """
type: command
short-summary: Deletes a server key.
"""

helps['sql server key show'] = """
type: command
short-summary: Shows a server key.
"""

helps['sql server list'] = """
type: command
short-summary: List available servers.
examples:
  - name: List all servers in the current subscription.
    text: az sql server list
  - name: List all servers in a resource group.
    text: az sql server list -g mygroup
"""

helps['sql server tde-key'] = """
type: group
short-summary: Manage a server's encryption protector.
"""

helps['sql server tde-key set'] = """
type: command
short-summary: Sets the server's encryption protector.
"""

helps['sql server update'] = """
type: command
short-summary: Update a server.
examples:
  - name: Update a server. (autogenerated)
    text: az sql server update --admin-password myadminpassword --name MyAzureSQLServer --resource-group MyResourceGroup
    crafted: true
  - name: Update a server with User Managed Identies and Identity Type is SystemAssigned,UserAssigned.
    text: az sql server update -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type SystemAssigned,UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi
  - name: Update a server with User Managed Identies and Identity Type is UserAssigned.
    text: az sql server update -g myResourceGroup -n myServer -i \\
              --user-assigned-identity-id /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi \\
              --identity-type UserAssigned --pid /subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testumi
"""

helps['sql server vnet-rule'] = """
type: group
short-summary: Manage a server's virtual network rules.
"""

helps['sql server vnet-rule create'] = """
type: command
short-summary: Create a virtual network rule to allows access to an Azure SQL server.

examples:
  - name: Create a vnet rule by providing the subnet id.
    text: |
        az sql server vnet-rule create --server MyAzureSqlServer --name MyVNetRule \\
          -g MyResourceGroup --subnet /subscriptions/{SubID}/resourceGroups/{ResourceGroup}/providers/Microsoft.Network/virtualNetworks/{VNETName}/subnets/{SubnetName}
  - name: Create a vnet rule by providing the vnet and subnet name. The subnet id is created by taking the resource group name and subscription id of the SQL server.
    text: |
        az sql server vnet-rule create --server MyAzureSqlServer --name MyVNetRule \\
            -g MyResourceGroup --subnet subnetName --vnet-name vnetName
"""

helps['sql server vnet-rule update'] = """
type: command
short-summary: Update a virtual network rule.
"""

helps['sql server wait'] = """
type: command
short-summary: Place the CLI in a waiting state until a condition of the SQL server is met.
"""

helps['sql virtual-cluster'] = """
type: group
short-summary: Manage SQL virtual clusters.
"""

helps['sql virtual-cluster delete'] = """
type: command
short-summary: Delete a virtual cluster.
examples:
  - name: Delete a virtual cluster
    text: az sql virtual-cluster delete -g mygroup -n mycluster
"""

helps['sql virtual-cluster list'] = """
type: command
short-summary: List available virtual clusters.
examples:
  - name: List all virtual clusters in the current subscription.
    text: az sql virtual-cluster list
  - name: List all virtual clusters in a resource group.
    text: az sql virtual-cluster list -g mygroup
"""

helps['sql virtual-cluster show'] = """
type: command
short-summary: Get the details for a virtual cluster.
examples:
  - name: Get the details for a virtual cluster
    text: az sql virtual-cluster show -g mygroup -n mycluster
"""

helps['sql db classification'] = """
type: group
short-summary: Manage sensitivity classifications.
"""

helps['sql db classification update'] = """
type: command
short-summary: Update a columns's sensitivity classification.
examples:
  - name: Update sensitivity classification for a given column.
    text: az sql db classification update -g mygroup -s myserver -n mydb --schema dbo --table mytable --column mycolumn --information-type Name --label "Confidential - GDPR"
"""

helps['sql db classification list'] = """
type: command
short-summary: Get the sensitivity classifications of a given database.
examples:
  - name: List the sensitivity classification of a given database.
    text: az sql db classification list -g mygroup -s myserver -n mydb
"""

helps['sql db classification show'] = """
type: command
short-summary: Get the sensitivity classification of a given column.
examples:
  - name: Get the sensitivity classification of a given column.
    text: az sql db classification show -g mygroup -s myserver -n mydb --schema dbo --table mytable --column mycolumn
"""

helps['sql db classification delete'] = """
type: command
short-summary: Delete the sensitivity classification of a given column.
examples:
  - name: Delete the sensitivity classification of a given column.
    text: az sql db classification delete -g mygroup -s myserver -n mydb --schema dbo --table mytable --column mycolumn
"""

helps['sql db classification recommendation'] = """
type: group
short-summary: Manage sensitivity classification recommendations.
"""

helps['sql db classification recommendation list'] = """
type: command
short-summary: List the recommended sensitivity classifications of a given database.
examples:
  - name: List the recommended sensitivity classifications of a given database.
    text: az sql db classification recommendation list -g mygroup -s myserver -n mydb
"""

helps['sql db classification recommendation enable'] = """
type: command
short-summary: Enable sensitivity recommendations for a given column (recommendations are enabled by default on all columns).
examples:
  - name: Enable sensitivity recommendations for a given column.
    text: az sql db classification recommendation enable -g mygroup -s myserver -n mydb --schema dbo --table mytable --column mycolumn
"""

helps['sql db classification recommendation disable'] = """
type: command
short-summary: Disable sensitivity recommendations for a given column (recommendations are enabled by default on all columns).
examples:
  - name: Disable sensitivity recommendations for a given column.
    text: az sql db classification recommendation disable -g mygroup -s myserver -n mydb --schema dbo --table mytable --column mycolumn
"""

helps['sql db ledger-digest-uploads'] = """
type: group
short-summary: Manage ledger digest upload settings.
"""

helps['sql db ledger-digest-uploads enable'] = """
type: command
short-summary: Enable uploading ledger digests to an Azure Storage account or to Azure Confidential Ledger. If uploading ledger digests is already enabled, the cmdlet resets the digest storage endpoint to a new value.
examples:
  - name: Enable uploading ledger digests to an Azure Blob storage.
    text: az sql db ledger-digest-uploads enable --name mydb --resource-group MyResourceGroup --server myserver --endpoint https://mystorage.blob.core.windows.net
"""

helps['sql db ledger-digest-uploads disable'] = """
type: command
short-summary: Disable uploading ledger digests.
examples:
  - name: Disable uploading ledger digests.
    text: az sql db ledger-digest-uploads disable --name mydb --resource-group MyResourceGroup --server myserver
"""

helps['sql db ledger-digest-uploads show'] = """
type: command
short-summary: Show the current ledger digest settings.
examples:
  - name: Show the settings for uploading ledger digests.
    text: az sql db ledger-digest-uploads show --name mydb --resource-group MyResourceGroup --server myserver
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import CliCommandType

from ._format import (
    db_list_transform,
    db_transform,
    db_table_format,
    db_edition_table_format,
    elastic_pool_list_transform,
    elastic_pool_transform,
    elastic_pool_table_format,
    elastic_pool_edition_table_format,
    firewall_rule_table_format,
    instance_pool_table_format,
    outbound_firewall_rule_table_format,
    server_table_format,
    usage_table_format,
    LongRunningOperationResultTransform
)

from ._util import (
    get_sql_backup_short_term_retention_policies_operations,
    get_sql_server_azure_ad_administrators_operations,
    get_sql_capabilities_operations,
    get_sql_databases_operations,
    get_sql_database_blob_auditing_policies_operations,
    get_sql_server_blob_auditing_policies_operations,
    get_sql_server_dev_ops_audit_settings_operations,
    get_sql_database_long_term_retention_backups_operations,
    get_sql_database_long_term_retention_policies_operations,
    get_sql_database_sensitivity_labels_operations,
    get_sql_database_operations_operations,
    get_sql_database_threat_detection_policies_operations,
    get_sql_database_transparent_data_encryptions_operations,
    get_sql_database_usages_operations,
    get_sql_elastic_pools_operations,
    get_sql_elastic_pool_operations_operations,
    get_sql_encryption_protectors_operations,
    get_sql_failover_groups_operations,
    get_sql_firewall_rules_operations,
    get_sql_outbound_firewall_rules_operations,
    get_sql_instance_pools_operations,
    get_sql_managed_databases_operations,
    get_sql_managed_database_restore_details_operations,
    get_sql_managed_backup_short_term_retention_policies_operations,
    get_sql_managed_database_long_term_retention_policies_operations,
    get_sql_managed_database_long_term_retention_backups_operations,
    get_sql_managed_instance_azure_ad_administrators_operations,
    get_sql_managed_instance_azure_ad_only_operations,
    get_sql_managed_instance_encryption_protectors_operations,
    get_sql_managed_instance_keys_operations,
    get_sql_managed_instance_operations_operations,
    get_sql_managed_instances_operations,
    get_sql_server_trust_groups_operations,
    get_sql_replication_links_operations,
    get_sql_restorable_dropped_databases_operations,
    get_sql_restorable_dropped_managed_databases_operations,
    get_sql_server_azure_ad_only_operations,
    get_sql_server_connection_policies_operations,
    get_sql_server_dns_aliases_operations,
    get_sql_server_keys_operations,
    get_sql_servers_operations,
    get_sql_server_usages_operations,
    get_sql_subscription_usages_operations,
    get_sql_virtual_clusters_operations,
    get_sql_virtual_network_rules_operations,
    get_sql_instance_failover_groups_operations,
    get_sql_database_ledger_digest_uploads_operations
)

from ._validators import (
    validate_subnet
)


# pylint: disable=too-many-statements,line-too-long,too-many-locals
def load_command_table(self, _):

    ###############################################
    #                sql list-usages              #
    ###############################################

    subscription_usages_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#SubscriptionUsagesOperations.{}',
        client_factory=get_sql_subscription_usages_operations)

    with self.command_group('sql',
                            subscription_usages_operations,
                            client_factory=get_sql_subscription_usages_operations) as g:

        g.command('list-usages', 'list_by_location',
                  table_transformer=usage_table_format)

        g.command('show-usage', 'get',
                  table_transformer=usage_table_format)

    ###############################################
    #                sql db                       #
    ###############################################

    with self.command_group('sql db') as g:
        g.custom_command('show-connection-string', 'db_show_conn_str')

    database_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#DatabasesOperations.{}',
        client_factory=get_sql_databases_operations)

    database_lro_transform = LongRunningOperationResultTransform(
        self.cli_ctx, db_transform)

    with self.command_group('sql db',
                            database_operations,
                            client_factory=get_sql_databases_operations) as g:

        g.custom_command('create', 'db_create',
                         supports_no_wait=True,
                         transform=database_lro_transform,
                         table_transformer=db_table_format)
        g.custom_command('copy', 'db_copy',
                         supports_no_wait=True,
                         transform=database_lro_transform,
                         table_transformer=db_table_format)
        g.custom_command('restore', 'db_restore',
                         supports_no_wait=True,
                         transform=database_lro_transform,
                         table_transformer=db_table_format)
        g.custom_command('rename', 'db_rename',
                         transform=database_lro_transform,
                         table_transformer=db_table_format)
        g.show_command('show', 'get',
                       transform=db_transform,
                       table_transformer=db_table_format)
        g.custom_command('list', 'db_list',
                         transform=db_list_transform,
                         table_transformer=db_table_format)
        g.command('delete', 'begin_delete',
                  confirmation=True,
                  supports_no_wait=True)
        g.generic_update_command('update',
                                 custom_func_name='db_update',
                                 setter_name='begin_create_or_update',
                                 supports_no_wait=True,
                                 transform=database_lro_transform,
                                 table_transformer=db_table_format)

        g.custom_command('export', 'db_export')
        g.custom_command('import', 'db_import')

    capabilities_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#CapabilitiesOperations.{}',
        client_factory=get_sql_capabilities_operations)

    with self.command_group('sql db',
                            capabilities_operations,
                            client_factory=get_sql_capabilities_operations) as g:

        g.custom_command(
            'list-editions',
            'db_list_capabilities',
            table_transformer=db_edition_table_format)

    with self.command_group('sql db replica',
                            database_operations,
                            client_factory=get_sql_databases_operations) as g:

        g.custom_command('create', 'db_create_replica',
                         supports_no_wait=True,
                         transform=database_lro_transform,
                         table_transformer=db_table_format)

    with self.command_group('sql dw',
                            database_operations,
                            client_factory=get_sql_databases_operations) as g:

        g.custom_command('create', 'dw_create',
                         supports_no_wait=True,
                         transform=database_lro_transform)
        g.show_command('show', 'get',
                       transform=db_transform)
        g.custom_command('list', 'dw_list',
                         transform=db_list_transform)
        g.command('delete', 'begin_delete',
                  confirmation=True,
                  supports_no_wait=True)
        g.custom_command('pause', 'dw_pause')
        g.custom_command('resume', 'dw_resume')
        g.generic_update_command('update',
                                 custom_func_name='dw_update',
                                 setter_name='begin_create_or_update',
                                 supports_no_wait=True,
                                 transform=database_lro_transform)

    database_operations_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#DatabaseOperationsOperations.{}',
        client_factory=get_sql_database_operations_operations)

    with self.command_group('sql db op', database_operations_operations) as g:

        g.command('list', 'list_by_database')
        g.command('cancel', 'cancel')

    transparent_data_encryptions_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#TransparentDataEncryptionsOperations.{}',
        client_factory=get_sql_database_transparent_data_encryptions_operations)

    with self.command_group('sql db tde',
                            transparent_data_encryptions_operations,
                            client_factory=get_sql_database_transparent_data_encryptions_operations) as g:

        g.custom_command('set', 'transparent_data_encryptions_set')
        g.custom_show_command('show', 'transparent_data_encryptions_get')

    replication_links_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ReplicationLinksOperations.{}',
        client_factory=get_sql_replication_links_operations)

    with self.command_group('sql db replica',
                            replication_links_operations,
                            client_factory=get_sql_replication_links_operations) as g:

        g.command('list-links', 'list_by_database')
        g.custom_command('delete-link', 'db_delete_replica_link',
                         confirmation=True)
        g.custom_command('set-primary', 'db_failover')

    restorable_dropped_databases_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#RestorableDroppedDatabasesOperations.{}',
        client_factory=get_sql_restorable_dropped_databases_operations)

    with self.command_group('sql db', restorable_dropped_databases_operations) as g:

        g.command('list-deleted', 'list_by_server')

    restorable_dropped_managed_databases_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#RestorableDroppedManagedDatabasesOperations.{}',
        client_factory=get_sql_restorable_dropped_managed_databases_operations)

    with self.command_group('sql midb',
                            restorable_dropped_managed_databases_operations,
                            client_factory=get_sql_restorable_dropped_managed_databases_operations) as g:

        g.command('list-deleted', 'list_by_instance')

    database_blob_auditing_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#DatabaseBlobAuditingPoliciesOperations.{}',
        client_factory=get_sql_database_blob_auditing_policies_operations)

    with self.command_group('sql db audit-policy',
                            database_blob_auditing_policies_operations,
                            client_factory=get_sql_database_blob_auditing_policies_operations) as g:

        g.custom_show_command('show', 'db_audit_policy_show')
        g.generic_update_command('update', custom_func_name='db_audit_policy_update')
        g.wait_command('wait')

    server_blob_auditing_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerBlobAuditingPoliciesOperations.{}',
        client_factory=get_sql_server_blob_auditing_policies_operations)

    with self.command_group('sql server audit-policy',
                            server_blob_auditing_policies_operations,
                            client_factory=get_sql_server_blob_auditing_policies_operations) as g:

        g.custom_show_command('show', 'server_audit_policy_show')
        g.generic_update_command('update',
                                 setter_name='begin_create_or_update',
                                 custom_func_name='server_audit_policy_update',
                                 supports_no_wait=True)
        g.wait_command('wait')

    server_dev_ops_audit_settings_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerDevOpsAuditSettingsOperations.{}',
        client_factory=get_sql_server_dev_ops_audit_settings_operations)

    with self.command_group('sql server ms-support audit-policy',
                            server_dev_ops_audit_settings_operations,
                            client_factory=get_sql_server_dev_ops_audit_settings_operations) as g:

        g.custom_show_command('show', 'server_ms_support_audit_policy_show')
        g.generic_update_command('update', custom_func_name='server_ms_support_audit_policy_update',
                                 setter_name='server_ms_support_audit_policy_set',
                                 setter_type=CliCommandType(operations_tmpl='azure.cli.command_modules.sql.custom#{}'),
                                 getter_name='server_ms_support_audit_policy_get',
                                 getter_type=CliCommandType(operations_tmpl='azure.cli.command_modules.sql.custom#{}'),
                                 supports_no_wait=True)
        g.custom_wait_command('wait', 'server_ms_support_audit_policy_get')

    ledger_digest_uploads_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#LedgerDigestUploadsOperations.{}',
        client_factory=get_sql_database_ledger_digest_uploads_operations)

    with self.command_group('sql db ledger-digest-uploads',
                            ledger_digest_uploads_operations,
                            client_factory=get_sql_database_ledger_digest_uploads_operations) as g:

        g.custom_show_command('show', 'ledger_digest_uploads_show')
        g.custom_command('enable', 'ledger_digest_uploads_enable')
        g.custom_command('disable', 'ledger_digest_uploads_disable')

    database_long_term_retention_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#LongTermRetentionPoliciesOperations.{}',
        client_factory=get_sql_database_long_term_retention_policies_operations)

    with self.command_group('sql db ltr-policy',
                            database_long_term_retention_policies_operations,
                            client_factory=get_sql_database_long_term_retention_policies_operations,
                            is_preview=True) as g:

        g.custom_command('set', 'update_long_term_retention')
        g.custom_show_command('show', 'get_long_term_retention')

    database_long_term_retention_backups_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#LongTermRetentionBackupsOperations.{}',
        client_factory=get_sql_database_long_term_retention_backups_operations)

    with self.command_group('sql db ltr-backup',
                            database_long_term_retention_backups_operations,
                            client_factory=get_sql_database_long_term_retention_backups_operations,
                            is_preview=True) as g:

        g.show_command('show', 'get')
        g.custom_command('list', 'list_long_term_retention_backups')
        g.command('delete', 'begin_delete', confirmation=True)

    with self.command_group('sql db ltr-backup',
                            database_operations,
                            client_factory=get_sql_databases_operations,
                            is_preview=True) as g:
        g.custom_command(
            'restore',
            'restore_long_term_retention_backup',
            supports_no_wait=True)
        g.wait_command('wait')

    backup_short_term_retention_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#BackupShortTermRetentionPoliciesOperations.{}',
        client_factory=get_sql_backup_short_term_retention_policies_operations)

    with self.command_group('sql db str-policy',
                            backup_short_term_retention_policies_operations,
                            client_factory=get_sql_backup_short_term_retention_policies_operations,
                            is_preview=True) as g:

        g.custom_command('set', 'update_short_term_retention', supports_no_wait=True)
        g.custom_show_command('show', 'get_short_term_retention')
        g.wait_command('wait')

    database_sensitivity_labels_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#SensitivityLabelsOperations.{}',
        client_factory=get_sql_database_sensitivity_labels_operations)

    with self.command_group('sql db classification',
                            database_sensitivity_labels_operations,
                            client_factory=get_sql_database_sensitivity_labels_operations) as g:

        g.command('list', 'list_current_by_database')
        g.custom_show_command('show', 'db_sensitivity_label_show')
        g.command('delete', 'delete')
        g.custom_command('update', 'db_sensitivity_label_update')

    with self.command_group('sql db classification recommendation',
                            database_sensitivity_labels_operations,
                            client_factory=get_sql_database_sensitivity_labels_operations) as g:

        g.command('list', 'list_recommended_by_database')
        g.command('enable', 'enable_recommendation')
        g.command('disable', 'disable_recommendation')

    database_threat_detection_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#DatabaseSecurityAlertPoliciesOperations.{}',
        client_factory=get_sql_database_threat_detection_policies_operations)

    database_threat_detection_policy_update_sdk = CliCommandType(
        operations_tmpl='azure.cli.command_modules.sql.custom#{}')

    with self.command_group('sql db threat-policy',
                            database_threat_detection_policies_operations,
                            client_factory=get_sql_database_threat_detection_policies_operations) as g:

        g.custom_show_command('show', 'db_threat_detection_policy_get')
        g.generic_update_command('update',
                                 getter_name='db_threat_detection_policy_get',
                                 getter_type=database_threat_detection_policy_update_sdk,
                                 setter_name='db_threat_detection_policy_update_setter',
                                 setter_type=database_threat_detection_policy_update_sdk,
                                 custom_func_name='db_threat_detection_policy_update')

    database_usages_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#DatabaseUsagesOperations.{}',
        client_factory=get_sql_database_usages_operations)

    with self.command_group('sql db', database_usages_operations) as g:

        g.command('list-usages', 'list_by_database')

    ###############################################
    #                sql elastic-pool             #
    ###############################################

    elastic_pools_ops = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ElasticPoolsOperations.{}',
        client_factory=get_sql_elastic_pools_operations)

    elastic_pool_lro_transform = LongRunningOperationResultTransform(
        self.cli_ctx, elastic_pool_transform)

    with self.command_group('sql elastic-pool',
                            elastic_pools_ops,
                            client_factory=get_sql_elastic_pools_operations) as g:

        g.custom_command('create', 'elastic_pool_create',
                         supports_no_wait=True,
                         transform=elastic_pool_lro_transform,
                         table_transformer=elastic_pool_table_format)
        g.command('delete', 'begin_delete',
                  supports_no_wait=True)
        g.show_command('show', 'get',
                       transform=elastic_pool_transform,
                       table_transformer=elastic_pool_table_format)
        g.command('list', 'list_by_server',
                  transform=elastic_pool_list_transform,
                  table_transformer=elastic_pool_table_format)
        g.generic_update_command('update',
                                 custom_func_name='elastic_pool_update',
                                 setter_name='begin_create_or_update',
                                 supports_no_wait=True,
                                 transform=elastic_pool_lro_transform,
                                 table_transformer=elastic_pool_table_format)

    with self.command_group('sql elastic-pool', database_operations) as g:

        g.command('list-dbs', 'list_by_elastic_pool',
                  transform=db_list_transform,
                  table_transformer=db_table_format)

    with self.command_group('sql elastic-pool',
                            capabilities_operations,
                            client_factory=get_sql_capabilities_operations) as g:

        g.custom_command('list-editions', 'elastic_pool_list_capabilities',
                         table_transformer=elastic_pool_edition_table_format)

    elastic_pool_operations_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ElasticPoolOperationsOperations.{}',
        client_factory=get_sql_elastic_pool_operations_operations)

    with self.command_group('sql elastic-pool op',
                            elastic_pool_operations_operations,
                            client_factory=get_sql_elastic_pool_operations_operations) as g:

        g.command('list', 'list_by_elastic_pool')
        g.command('cancel', 'cancel')

    ###############################################
    #             sql failover-group              #
    ###############################################

    failover_groups_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#FailoverGroupsOperations.{}',
        client_factory=get_sql_failover_groups_operations)
    with self.command_group('sql failover-group', failover_groups_operations, client_factory=get_sql_failover_groups_operations) as g:
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')
        g.custom_command('create', 'failover_group_create')
        g.generic_update_command('update',
                                 setter_name='begin_create_or_update',
                                 custom_func_name='failover_group_update')
        g.command('delete', 'begin_delete')
        g.custom_command('set-primary', 'failover_group_failover')

    ###############################################
    #             sql instance-pool               #
    ###############################################

    instance_pools_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#InstancePoolsOperations.{}',
        client_factory=get_sql_instance_pools_operations)
    with self.command_group('sql instance-pool', instance_pools_operations, client_factory=get_sql_instance_pools_operations, is_preview=True) as g:
        g.show_command('show', 'get',
                       table_transformer=instance_pool_table_format)
        g.custom_command('list', 'instance_pool_list',
                         table_transformer=instance_pool_table_format)
        g.generic_update_command('update',
                                 custom_func_name='instance_pool_update',
                                 setter_name='begin_update')
        g.command('delete', 'begin_delete', supports_no_wait=True, confirmation=True)
        g.custom_command('create', 'instance_pool_create',
                         supports_no_wait=True, table_transformer=instance_pool_table_format)
        g.wait_command('wait')

    ###############################################
    #                sql server                   #
    ###############################################

    servers_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServersOperations.{}',
        client_factory=get_sql_servers_operations)

    with self.command_group('sql server',
                            servers_operations,
                            client_factory=get_sql_servers_operations) as g:

        g.custom_command('create', 'server_create',
                         table_transformer=server_table_format,
                         supports_no_wait=True)
        g.command('delete', 'begin_delete',
                  confirmation=True)
        g.custom_show_command('show', 'server_get',
                              table_transformer=server_table_format)
        g.custom_command('list', 'server_list',
                         table_transformer=server_table_format)
        g.generic_update_command('update',
                                 custom_func_name='server_update',
                                 setter_name='begin_create_or_update',
                                 supports_no_wait=True)
        g.wait_command('wait')

    server_usages_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerUsagesOperations.{}',
        client_factory=get_sql_server_usages_operations)

    with self.command_group('sql server', server_usages_operations) as g:
        g.command('list-usages', 'list_by_server')

    firewall_rules_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#FirewallRulesOperations.{}',
        client_factory=get_sql_firewall_rules_operations)

    with self.command_group('sql server firewall-rule',
                            firewall_rules_operations,
                            client_factory=get_sql_firewall_rules_operations) as g:

        g.custom_command('create', 'firewall_rule_create',
                         table_transformer=firewall_rule_table_format)
        g.custom_command('update', 'firewall_rule_update',
                         table_transformer=firewall_rule_table_format)
        g.command('delete', 'delete')
        g.show_command('show', 'get',
                       table_transformer=firewall_rule_table_format)
        g.command('list', 'list_by_server',
                  table_transformer=firewall_rule_table_format)

    outbound_firewall_rules_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#OutboundFirewallRulesOperations.{}',
        client_factory=get_sql_outbound_firewall_rules_operations)

    with self.command_group('sql server outbound-firewall-rule',
                            outbound_firewall_rules_operations,
                            client_factory=get_sql_outbound_firewall_rules_operations) as g:
        g.custom_command('create', 'outbound_firewall_rule_create',
                         table_transformer=outbound_firewall_rule_table_format)
        g.command('delete', 'begin_delete')
        g.show_command('show', 'get',
                       table_transformer=outbound_firewall_rule_table_format)
        g.command('list', 'list_by_server',
                  table_transformer=outbound_firewall_rule_table_format)

    aadadmin_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerAzureADAdministratorsOperations.{}',
        client_factory=get_sql_server_azure_ad_administrators_operations)

    aadadmin_update_server_sdk = CliCommandType(
        operations_tmpl="azure.cli.command_modules.sql.custom#{}")

    with self.command_group('sql server ad-admin',
                            aadadmin_operations,
                            client_factory=get_sql_server_azure_ad_administrators_operations) as g:
        g.custom_command('create', 'server_ad_admin_set')
        g.command('list', 'list_by_server')
        g.custom_command('delete', 'server_ad_admin_delete')
        g.generic_update_command('update',
                                 getter_name='server_ad_admin_update_getter',
                                 getter_type=aadadmin_update_server_sdk,
                                 setter_name='server_ad_admin_update_setter',
                                 setter_type=aadadmin_update_server_sdk,
                                 custom_func_name='server_ad_admin_update')

    server_keys_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerKeysOperations.{}',
        client_factory=get_sql_server_keys_operations)

    with self.command_group('sql server key',
                            server_keys_operations,
                            client_factory=get_sql_server_keys_operations) as g:

        g.custom_command('create', 'server_key_create')
        g.custom_command('delete', 'server_key_delete')
        g.custom_show_command('show', 'server_key_get')
        g.command('list', 'list_by_server')

    encryption_protectors_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#EncryptionProtectorsOperations.{}',
        client_factory=get_sql_encryption_protectors_operations)

    with self.command_group('sql server tde-key',
                            encryption_protectors_operations,
                            client_factory=get_sql_encryption_protectors_operations) as g:

        g.custom_show_command('show', 'encryption_protector_get')
        g.custom_command('set', 'encryption_protector_update')

    virtual_network_rules_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#VirtualNetworkRulesOperations.{}',
        client_factory=get_sql_virtual_network_rules_operations)

    with self.command_group('sql server vnet-rule',
                            virtual_network_rules_operations,
                            client_factory=get_sql_virtual_network_rules_operations) as g:

        g.custom_command('create',
                         'vnet_rule_begin_create_or_update',
                         validator=validate_subnet)
        g.show_command('show', 'get')
        g.command('list', 'list_by_server')
        g.command('delete', 'begin_delete')
        g.custom_command('update',
                         'vnet_rule_begin_create_or_update')

    server_connection_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerConnectionPoliciesOperations.{}',
        client_factory=get_sql_server_connection_policies_operations)

    with self.command_group('sql server conn-policy',
                            server_connection_policies_operations,
                            client_factory=get_sql_server_connection_policies_operations) as c:

        c.custom_show_command('show', 'show_conn_policy')
        c.custom_command('update', 'update_conn_policy')

    server_dns_aliases_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerDnsAliasesOperations.{}',
        client_factory=get_sql_server_dns_aliases_operations)

    with self.command_group('sql server dns-alias',
                            server_dns_aliases_operations,
                            client_factory=get_sql_server_dns_aliases_operations) as c:

        c.show_command('show', 'get')
        c.command('list', 'list_by_server')
        c.command('create', 'begin_create_or_update')
        c.command('delete', 'begin_delete')
        c.custom_command('set', 'server_dns_alias_set')

    server_aadonly_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerAzureADOnlyAuthenticationsOperations.{}',
        client_factory=get_sql_server_azure_ad_only_operations)

    with self.command_group('sql server ad-only-auth',
                            server_aadonly_operations,
                            client_factory=get_sql_server_azure_ad_only_operations) as g:

        g.custom_command('disable', 'server_aad_only_disable')
        g.custom_command('enable', 'server_aad_only_enable')
        g.custom_command('get', 'server_aad_only_get')

    ###############################################
    #           sql server trust groups           #
    ###############################################

    server_trust_groups_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ServerTrustGroupsOperations.{}',
        client_factory=get_sql_server_trust_groups_operations)

    with self.command_group('sql stg', server_trust_groups_operations, client_factory=get_sql_server_trust_groups_operations, is_preview=True) as g:
        g.custom_command('create', 'server_trust_group_create', supports_no_wait=True)
        g.custom_command('delete', 'server_trust_group_delete', confirmation=True, supports_no_wait=True)
        g.custom_show_command('show', 'server_trust_group_get')
        g.custom_command('list', 'server_trust_group_list')

    ###############################################
    #                sql managed instance         #
    ###############################################

    managed_instance_operations_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedInstanceOperationsOperations.{}',
        client_factory=get_sql_managed_instance_operations_operations)

    with self.command_group('sql mi op', managed_instance_operations_operations) as g:

        g.command('list', 'list_by_managed_instance')
        g.show_command('show', 'get')
        g.command('cancel', 'cancel')

    managed_instances_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedInstancesOperations.{}',
        client_factory=get_sql_managed_instances_operations)

    with self.command_group('sql mi',
                            managed_instances_operations,
                            client_factory=get_sql_managed_instances_operations) as g:

        g.custom_command('create', 'managed_instance_create', supports_no_wait=True)
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.custom_show_command('show', 'managed_instance_get')
        g.custom_command('list', 'managed_instance_list')
        g.generic_update_command('update',
                                 setter_name='begin_create_or_update',
                                 custom_func_name='managed_instance_update',
                                 supports_no_wait=True)
        g.command('failover', 'begin_failover', supports_no_wait=True)

    managed_instance_keys_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedInstanceKeysOperations.{}',
        client_factory=get_sql_managed_instance_keys_operations)

    with self.command_group('sql mi key',
                            managed_instance_keys_operations,
                            client_factory=get_sql_managed_instance_keys_operations) as g:

        g.custom_command('create', 'managed_instance_key_create')
        g.custom_command('delete', 'managed_instance_key_delete')
        g.custom_show_command('show', 'managed_instance_key_get')
        g.command('list', 'list_by_instance')

    managed_instance_encryption_protectors_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedInstanceEncryptionProtectorsOperations.{}',
        client_factory=get_sql_managed_instance_encryption_protectors_operations)

    with self.command_group('sql mi tde-key',
                            managed_instance_encryption_protectors_operations,
                            client_factory=get_sql_managed_instance_encryption_protectors_operations) as g:

        g.custom_show_command('show', 'managed_instance_encryption_protector_get')
        g.custom_command('set', 'managed_instance_encryption_protector_update')

    managed_instance_aadadmin_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedInstanceAdministratorsOperations.{}',
        client_factory=get_sql_managed_instance_azure_ad_administrators_operations)

    with self.command_group('sql mi ad-admin',
                            managed_instance_aadadmin_operations,
                            client_factory=get_sql_managed_instance_azure_ad_administrators_operations) as g:

        g.custom_command('create', 'mi_ad_admin_set')
        g.command('list', 'list_by_instance')
        g.custom_command('delete', 'mi_ad_admin_delete')
        g.custom_command('update', 'mi_ad_admin_set')

    managed_instance_aadonly_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedInstanceAzureADOnlyAuthenticationsOperations.{}',
        client_factory=get_sql_managed_instance_azure_ad_only_operations)

    with self.command_group('sql mi ad-only-auth',
                            managed_instance_aadonly_operations,
                            client_factory=get_sql_managed_instance_azure_ad_only_operations) as g:

        g.custom_command('disable', 'mi_aad_only_disable')
        g.custom_command('enable', 'mi_aad_only_enable')
        g.custom_command('get', 'mi_aad_only_get')

    ###############################################
    #                sql managed db               #
    ###############################################

    managed_databases_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedDatabasesOperations.{}',
        client_factory=get_sql_managed_databases_operations)

    with self.command_group('sql midb',
                            managed_databases_operations,
                            client_factory=get_sql_managed_databases_operations) as g:

        g.custom_command('create', 'managed_db_create', supports_no_wait=True)
        g.generic_update_command('update',
                                 setter_name='begin_create_or_update',
                                 custom_func_name='managed_db_update',
                                 supports_no_wait=True)
        g.custom_command('restore', 'managed_db_restore', supports_no_wait=True)
        g.show_command('show', 'get')
        g.command('list', 'list_by_instance')
        g.command('delete', 'begin_delete', confirmation=True, supports_no_wait=True)

    managed_backup_short_term_retention_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedBackupShortTermRetentionPoliciesOperations.{}',
        client_factory=get_sql_managed_backup_short_term_retention_policies_operations)

    with self.command_group('sql midb short-term-retention-policy',
                            managed_backup_short_term_retention_policies_operations,
                            client_factory=get_sql_managed_backup_short_term_retention_policies_operations) as g:

        g.custom_command(
            'set',
            'update_short_term_retention_mi',
            supports_no_wait=True,
            is_preview=True)
        g.custom_show_command('show', 'get_short_term_retention_mi', is_preview=True)

    managed_database_long_term_retention_policies_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedInstanceLongTermRetentionPoliciesOperations.{}',
        client_factory=get_sql_managed_database_long_term_retention_policies_operations)

    with self.command_group('sql midb ltr-policy',
                            managed_database_long_term_retention_policies_operations,
                            client_factory=get_sql_managed_database_long_term_retention_policies_operations) as g:

        g.custom_command('set', 'update_long_term_retention_mi', is_preview=True)
        g.custom_show_command('show', 'get_long_term_retention_mi', is_preview=True)

    managed_database_long_term_retention_backups_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#LongTermRetentionManagedInstanceBackupsOperations.{}',
        client_factory=get_sql_managed_database_long_term_retention_backups_operations)

    with self.command_group('sql midb ltr-backup',
                            managed_database_long_term_retention_backups_operations,
                            client_factory=get_sql_managed_database_long_term_retention_backups_operations) as g:
        g.custom_show_command('show', 'get_long_term_retention_mi_backup', is_preview=True)
        g.custom_command(
            'list',
            'list_long_term_retention_mi_backups', is_preview=True)
        g.custom_command('delete', 'delete_long_term_retention_mi_backup', confirmation=True, is_preview=True)

    with self.command_group('sql midb ltr-backup',
                            managed_databases_operations,
                            client_factory=get_sql_managed_databases_operations) as g:
        g.custom_command(
            'restore',
            'restore_long_term_retention_mi_backup',
            supports_no_wait=True,
            is_preview=True)
        g.wait_command('wait')

    with self.command_group('sql midb log-replay',
                            managed_databases_operations,
                            client_factory=get_sql_managed_databases_operations) as g:
        g.custom_command('start', 'managed_db_log_replay_start', supports_no_wait=True)
        g.command('stop', 'begin_delete', confirmation=True, supports_no_wait=True)
        g.custom_command('complete', 'managed_db_log_replay_complete_restore')
        g.wait_command('wait')

    managed_database_restore_details_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#ManagedDatabaseRestoreDetailsOperations.{}',
        client_factory=get_sql_managed_database_restore_details_operations)

    with self.command_group('sql midb log-replay',
                            managed_database_restore_details_operations,
                            client_factory=get_sql_managed_database_restore_details_operations) as g:

        g.custom_show_command('show', 'managed_db_log_replay_get')

    ###############################################
    #                sql virtual cluster         #
    ###############################################

    virtual_clusters_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#VirtualClustersOperations.{}',
        client_factory=get_sql_virtual_clusters_operations)

    with self.command_group('sql virtual-cluster',
                            virtual_clusters_operations,
                            client_factory=get_sql_virtual_clusters_operations) as g:

        g.command('delete', 'begin_delete', supports_no_wait=True)
        g.show_command('show', 'get')
        g.custom_command('list', 'virtual_cluster_list')

    ###############################################
    #             sql instance failover group     #
    ###############################################

    instance_failover_groups_operations = CliCommandType(
        operations_tmpl='azure.mgmt.sql.operations#InstanceFailoverGroupsOperations.{}',
        client_factory=get_sql_instance_failover_groups_operations)
    with self.command_group('sql instance-failover-group', instance_failover_groups_operations, client_factory=get_sql_instance_failover_groups_operations) as g:
        g.show_command('show', 'get')
        g.custom_command('create', 'instance_failover_group_create')
        g.generic_update_command('update',
                                 setter_name='begin_create_or_update',
                                 custom_func_name='instance_failover_group_update')
        g.command('delete', 'begin_delete')
        g.custom_command('set-primary', 'instance_failover_group_failover')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long
# pylint: disable=too-many-statements

from azure.cli.core.commands.parameters import tags_type, get_enum_type, resource_group_name_type, name_type, get_location_type, get_resource_name_completion_list, get_three_state_flag
from azure.cli.core.commands.validators import get_default_location_from_resource_group


def load_arguments_sb(self, _):
    from azure.cli.command_modules.relay._completers import get_wcfrelay_command_completion_list, \
        get_hyco_command_completion_list

    from knack.arguments import CLIArgumentType
    from azure.mgmt.relay.models import SkuTier, AccessRights, KeyType
    rights_arg_type = CLIArgumentType(options_list=['--rights'], nargs='+', arg_type=get_enum_type(AccessRights), help='Space-separated list of Authorization rule rights')
    key_arg_type = CLIArgumentType(options_list=['--key'], arg_type=get_enum_type(KeyType), help='specifies Primary or Secondary key needs to be reset')
    keyvalue_arg_type = CLIArgumentType(options_list=['--key-value'], help='Optional, if the key value provided, is set for KeyType or autogenerated Key value set for keyType.')

    with self.argument_context('relay') as c:
        c.argument('resource_group_name', arg_type=resource_group_name_type)
        c.argument('namespace_name', options_list=['--namespace-name'], id_part='name', help='Name of Namespace')

    with self.argument_context('relay namespace') as c:
        c.argument('namespace_name', id_part='name', arg_type=name_type, completer=get_resource_name_completion_list('Microsoft.relay/namespaces'), help='Name of Namespace')

    with self.argument_context('relay namespace exists') as c:
        c.argument('name', arg_type=name_type, help='Namespace name. Name can contain only letters, numbers, and hyphens. The namespace must start with a letter, and it must end with a letter or number.')

    for scope in ['relay namespace create', 'relay namespace update']:
        with self.argument_context(scope) as c:
            c.argument('tags', arg_type=tags_type)
            c.argument('sku', arg_type=get_enum_type(SkuTier))

    with self.argument_context('relay namespace create') as c:
        c.argument('location', arg_type=get_location_type(self.cli_ctx), validator=get_default_location_from_resource_group)

    # region Namespace Authorization Rule
    with self.argument_context('relay namespace authorization-rule list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

    with self.argument_context('relay namespace authorization-rule') as c:
        c.argument('authorization_rule_name', arg_type=name_type, id_part='child_name_1', help='Name of Namespace Authorization Rule')
        c.argument('namespace_name', id_part='name', options_list=['--namespace-name'], help='Name of Namespace')

    for scope in ['relay namespace authorization-rule create', 'relay namespace authorization-rule update', 'relay wcfrelay authorization-rule create', 'relay wcfrelay authorization-rule update', 'relay hyco authorization-rule create', 'relay hyco authorization-rule update']:
        with self.argument_context(scope) as c:
            c.argument('rights', arg_type=rights_arg_type)

    with self.argument_context('relay namespace authorization-rule keys renew') as c:
        c.argument('key_type', arg_type=key_arg_type)
        c.argument('key', arg_type=keyvalue_arg_type)

    with self.argument_context('relay namespace authorization-rule keys list') as c:
        c.argument('authorization_rule_name', arg_type=name_type, id_part=None, help='Name of Namespace Authorization Rule')
        c.argument('namespace_name', id_part=None, options_list=['--namespace-name'], help='Name of Namespace')

    # region WCF Relay
    with self.argument_context('relay wcfrelay') as c:
        c.argument('relay_name', arg_type=name_type, id_part='child_name_1', completer=get_wcfrelay_command_completion_list, help='Name of WCF Relay')

    for scope in ['relay wcfrelay create', 'relay wcfrelay update']:
        with self.argument_context(scope) as c:
            c.argument('status', arg_type=get_enum_type(['Active', 'Disabled', 'SendDisabled', 'ReceiveDisabled']), help='Enumerates the possible values for the status of a messaging entity.')
            c.argument('relay_type', arg_type=get_enum_type(['Http', 'NetTcp']), default='NetTcp', help='Relay type')
            c.argument('user_metadata', help='Endpoint metadata')

    with self.argument_context('relay wcfrelay create') as c:
        c.argument('requires_client_authorization', arg_type=get_three_state_flag(), help='Indicates whether client authorization is required')
        c.argument('requires_transport_security', arg_type=get_three_state_flag(), help='Indicates whether transport security is required')

    with self.argument_context('relay wcfrelay list') as c:
        c.argument('namespace_name', id_part=None, options_list=['--namespace-name'], help='Name of Namespace')

    # region WCF Relay Authorization Rule
    with self.argument_context('relay wcfrelay authorization-rule') as c:
        c.argument('authorization_rule_name', arg_type=name_type, id_part='child_name_2', help='Name of WCF Relay Authorization Rule')
        c.argument('relay_name', id_part='child_name_1', options_list=['--relay-name'], help='Name of WCF Relay')

    with self.argument_context('relay wcfrelay authorization-rule list') as c:
        c.argument('namespace_name', id_part=None, options_list=['--namespace-name'], help='Name of Namespace')
        c.argument('relay_name', id_part=None, options_list=['--relay-name'], help='Name of WCF Relay')

    with self.argument_context('relay wcfrelay authorization-rule keys renew') as c:
        c.argument('key_type', arg_type=key_arg_type)
        c.argument('key', arg_type=keyvalue_arg_type)

    with self.argument_context('relay wcfrelay authorization-rule keys list') as c:
        c.argument('authorization_rule_name', arg_type=name_type, id_part=None, help='Name of WCF Relay Authorization Rule')
        c.argument('relay_name', id_part=None, options_list=['--relay-name'], help='Name of WCF Relay')
        c.argument('namespace_name', id_part=None, options_list=['--namespace-name'], help='Name of Namespace')

    # region - Hybrid Connection
    for scope in ['relay hyco show', 'relay hyco delete']:
        with self.argument_context(scope) as c:
            c.argument('hybrid_connection_name', arg_type=name_type, id_part='child_name_1', completer=get_hyco_command_completion_list, help='Name of Hybrid Connection')

    # region - Hybrid Connection Create
    with self.argument_context('relay hyco create') as c:
        c.argument('hybrid_connection_name', arg_type=name_type, id_part='child_name_1', completer=get_hyco_command_completion_list, help='Name of Hybrid Connection')
        c.argument('status', arg_type=get_enum_type(['Active', 'Disabled', 'SendDisabled', 'ReceiveDisabled']), help='Enumerates the possible values for the status of a messaging entity.')
        c.argument('requires_client_authorization', arg_type=get_three_state_flag(), help='Indicates whether client authorization is required')
        c.argument('user_metadata', help='Endpoint metadata')

    # region - Hybrid Connection Update
    with self.argument_context('relay hyco update') as c:
        c.argument('hybrid_connection_name', arg_type=name_type, id_part='child_name_1', completer=get_hyco_command_completion_list, help='Name of Hybrid Connection')
        c.argument('status', arg_type=get_enum_type(['Active', 'Disabled', 'SendDisabled', 'ReceiveDisabled']), help='Enumerates the possible values for the status of a messaging entity.')
        c.argument('requires_client_authorization', arg_type=get_three_state_flag(), help='Indicates whether client authorization is required')
        c.argument('user_metadata', help='Endpoint metadata')

    with self.argument_context('relay hyco list') as c:
        c.argument('namespace_name', id_part=None, options_list=['--namespace-name'], help='Name of Namespace')

    # region Hybrid Connection Authorization Rule
    with self.argument_context('relay hyco authorization-rule') as c:
        c.argument('authorization_rule_name', arg_type=name_type, id_part='child_name_2', help='name of Hybrid Connection Authorization Rule')
        c.argument('hybrid_connection_name', options_list=['--hybrid-connection-name'], id_part='child_name_1', help='name of Hybrid Connection')

    with self.argument_context('relay hyco authorization-rule list') as c:
        c.argument('namespace_name', id_part=None, options_list=['--namespace-name'], help='Name of Namespace')
        c.argument('hybrid_connection_name', options_list=['--hybrid-connection-name'], id_part=None, help='name of Hybrid Connection')

    with self.argument_context('relay hyco authorization-rule keys list') as c:
        c.argument('authorization_rule_name', arg_type=name_type, id_part=None, help='Name of Hybrid Connection Authorization Rule')
        c.argument('hybrid_connection_name', id_part=None, options_list=['--hybrid-connection-name'], help='Name of Hybrid Connection')
        c.argument('namespace_name', id_part=None, options_list=['--namespace-name'], help='Name of Namespace')

    with self.argument_context('relay hyco authorization-rule keys renew') as c:
        c.argument('key_type', arg_type=key_arg_type)
        c.argument('key', arg_type=keyvalue_arg_type)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import random
import os

import azure.mgmt.redhatopenshift.models as openshiftcluster

from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.commands.client_factory import get_subscription_id
from azure.cli.core.profiles import ResourceType
from azure.cli.core.util import sdk_no_wait
from azure.cli.core.azclierror import ResourceNotFoundError, UnauthorizedError
from azure.cli.command_modules.aro._aad import AADManager
from azure.cli.command_modules.aro._rbac import assign_role_to_resource, \
    has_role_assignment_on_resource
from azure.cli.command_modules.aro._rbac import ROLE_NETWORK_CONTRIBUTOR, ROLE_READER
from azure.cli.command_modules.aro._validators import validate_subnets
from azure.cli.command_modules.role import GraphError

from knack.log import get_logger

from msrestazure.azure_exceptions import CloudError
from msrestazure.tools import resource_id, parse_resource_id
from msrest.exceptions import HttpOperationError

logger = get_logger(__name__)

FP_CLIENT_ID = 'f1dd0a37-89c6-4e07-bcd1-ffd3d43d8875'


def aro_create(cmd,  # pylint: disable=too-many-locals
               client,
               resource_group_name,
               resource_name,
               master_subnet,
               worker_subnet,
               vnet=None,  # pylint: disable=unused-argument
               vnet_resource_group_name=None,  # pylint: disable=unused-argument
               location=None,
               pull_secret=None,
               domain=None,
               cluster_resource_group=None,
               fips_validated_modules=None,
               client_id=None,
               client_secret=None,
               pod_cidr=None,
               service_cidr=None,
               disk_encryption_set=None,
               master_encryption_at_host=False,
               master_vm_size=None,
               worker_encryption_at_host=False,
               worker_vm_size=None,
               worker_vm_disk_size_gb=None,
               worker_count=None,
               apiserver_visibility=None,
               ingress_visibility=None,
               tags=None,
               no_wait=False):

    resource_client = get_mgmt_service_client(
        cmd.cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)
    provider = resource_client.providers.get('Microsoft.RedHatOpenShift')
    if provider.registration_state != 'Registered':
        raise UnauthorizedError('Microsoft.RedHatOpenShift provider is not registered.',
                                'Run `az provider register -n Microsoft.RedHatOpenShift --wait`.')

    validate_subnets(master_subnet, worker_subnet)

    subscription_id = get_subscription_id(cmd.cli_ctx)

    random_id = generate_random_id()

    aad = AADManager(cmd.cli_ctx)
    if client_id is None:
        client_id, client_secret = aad.create_application(cluster_resource_group or 'aro-' + random_id)

    client_sp_id = aad.get_service_principal_id(client_id)
    if not client_sp_id:
        client_sp_id = aad.create_service_principal(client_id)

    rp_client_sp_id = aad.get_service_principal_id(resolve_rp_client_id())
    if not rp_client_sp_id:
        raise ResourceNotFoundError("RP service principal not found.")

    worker_vm_size = worker_vm_size or 'Standard_D4s_v3'

    if apiserver_visibility is not None:
        apiserver_visibility = apiserver_visibility.capitalize()

    if ingress_visibility is not None:
        ingress_visibility = ingress_visibility.capitalize()

    oc = openshiftcluster.OpenShiftCluster(
        location=location,
        tags=tags,
        cluster_profile=openshiftcluster.ClusterProfile(
            pull_secret=pull_secret or "",
            domain=domain or random_id,
            resource_group_id=(f"/subscriptions/{subscription_id}"
                               f"/resourceGroups/{cluster_resource_group or 'aro-' + random_id}"),
            fips_validated_modules='Enabled' if fips_validated_modules else 'Disabled',
        ),
        service_principal_profile=openshiftcluster.ServicePrincipalProfile(
            client_id=client_id,
            client_secret=client_secret,
        ),
        network_profile=openshiftcluster.NetworkProfile(
            pod_cidr=pod_cidr or '10.128.0.0/14',
            service_cidr=service_cidr or '172.30.0.0/16',
        ),
        master_profile=openshiftcluster.MasterProfile(
            vm_size=master_vm_size or 'Standard_D8s_v3',
            subnet_id=master_subnet,
            encryption_at_host='Enabled' if master_encryption_at_host else 'Disabled',
            disk_encryption_set_id=disk_encryption_set,
        ),
        worker_profiles=[
            openshiftcluster.WorkerProfile(
                name='worker',  # TODO: 'worker' should not be hard-coded
                vm_size=worker_vm_size,
                disk_size_gb=worker_vm_disk_size_gb or 128,
                subnet_id=worker_subnet,
                count=worker_count or 3,
                encryption_at_host='Enabled' if worker_encryption_at_host else 'Disabled',
                disk_encryption_set_id=disk_encryption_set,
            )
        ],
        apiserver_profile=openshiftcluster.APIServerProfile(
            visibility=apiserver_visibility or 'Public',
        ),
        ingress_profiles=[
            openshiftcluster.IngressProfile(
                name='default',  # TODO: 'default' should not be hard-coded
                visibility=ingress_visibility or 'Public',
            )
        ],
    )

    sp_obj_ids = [client_sp_id, rp_client_sp_id]
    ensure_resource_permissions(cmd.cli_ctx, oc, True, sp_obj_ids)

    return sdk_no_wait(no_wait, client.begin_create_or_update,
                       resource_group_name=resource_group_name,
                       resource_name=resource_name,
                       parameters=oc)


def aro_delete(cmd, client, resource_group_name, resource_name, no_wait=False):
    # TODO: clean up rbac
    rp_client_sp_id = None

    try:
        oc = client.get(resource_group_name, resource_name)
    except CloudError as e:
        if e.status_code == 404:
            raise ResourceNotFoundError(e.message) from e
        logger.info(e.message)
    except HttpOperationError as e:
        logger.info(e.message)

    aad = AADManager(cmd.cli_ctx)

    # Best effort - assume the role assignments on the SP exist if exception raised
    try:
        rp_client_sp_id = aad.get_service_principal_id(resolve_rp_client_id())
        if not rp_client_sp_id:
            raise ResourceNotFoundError("RP service principal not found.")
    except GraphError as e:
        logger.info(e.message)

    # Customers frequently remove the Cluster or RP's service principal permissions.
    # Attempt to fix this before performing any action against the cluster
    if rp_client_sp_id:
        ensure_resource_permissions(cmd.cli_ctx, oc, False, [rp_client_sp_id])

    return sdk_no_wait(no_wait, client.begin_delete,
                       resource_group_name=resource_group_name,
                       resource_name=resource_name)


def aro_list(client, resource_group_name=None):
    if resource_group_name:
        return client.list_by_resource_group(resource_group_name)
    return client.list()


def aro_show(client, resource_group_name, resource_name):
    return client.get(resource_group_name, resource_name)


def aro_list_credentials(client, resource_group_name, resource_name):
    return client.list_credentials(resource_group_name, resource_name)


def aro_update(cmd,
               client,
               resource_group_name,
               resource_name,
               refresh_cluster_credentials=False,
               client_id=None,
               client_secret=None,
               no_wait=False):
    # if we can't read cluster spec, we will not be able to do much. Fail.
    oc = client.get(resource_group_name, resource_name)

    ocUpdate = openshiftcluster.OpenShiftClusterUpdate()

    client_id, client_secret = cluster_application_update(cmd.cli_ctx, oc, client_id, client_secret, refresh_cluster_credentials)  # pylint: disable=line-too-long

    if client_id is not None or client_secret is not None:
        # construct update payload
        ocUpdate.service_principal_profile = openshiftcluster.ServicePrincipalProfile()

        if client_secret is not None:
            ocUpdate.service_principal_profile.client_secret = client_secret

        if client_id is not None:
            ocUpdate.service_principal_profile.client_id = client_id

    return sdk_no_wait(no_wait, client.begin_update,
                       resource_group_name=resource_group_name,
                       resource_name=resource_name,
                       parameters=ocUpdate)


def rp_mode_development():
    return os.environ.get('RP_MODE', '').lower() == 'development'


def rp_mode_production():
    return os.environ.get('RP_MODE', '') == ''


def generate_random_id():
    random_id = (random.choice('abcdefghijklmnopqrstuvwxyz') +
                 ''.join(random.choice('abcdefghijklmnopqrstuvwxyz1234567890')
                         for _ in range(7)))
    return random_id


def get_route_tables_from_subnets(cli_ctx, subnets):
    network_client = get_mgmt_service_client(cli_ctx, ResourceType.MGMT_NETWORK)

    route_tables = set()
    for sn in subnets:
        sid = parse_resource_id(sn)

        subnet = network_client.subnets.get(resource_group_name=sid['resource_group'],
                                            virtual_network_name=sid['name'],
                                            subnet_name=sid['resource_name'])

        if subnet.route_table is not None:
            route_tables.add(subnet.route_table.id)

    return route_tables


def get_cluster_network_resources(cli_ctx, oc):
    master_subnet = oc.master_profile.subnet_id
    worker_subnets = set()

    # Ensure that worker_profiles exists
    # it will not be returned if the cluster resources do not exist
    if oc.worker_profiles is not None:
        worker_subnets = {w.subnet_id for w in oc.worker_profiles}

    master_parts = parse_resource_id(master_subnet)
    vnet = resource_id(
        subscription=master_parts['subscription'],
        resource_group=master_parts['resource_group'],
        namespace='Microsoft.Network',
        type='virtualNetworks',
        name=master_parts['name'],
    )

    return get_network_resources(cli_ctx, worker_subnets | {master_subnet}, vnet)


def get_network_resources(cli_ctx, subnets, vnet):
    route_tables = get_route_tables_from_subnets(cli_ctx, subnets)

    resources = set()
    resources.add(vnet)
    resources.update(route_tables)

    return resources


def get_disk_encryption_resources(oc):
    disk_encryption_set = oc.master_profile.disk_encryption_set_id
    resources = set()
    if disk_encryption_set:
        resources.add(disk_encryption_set)
    return resources


# cluster_application_update manages cluster application & service principal update
# If called without parameters it should be best-effort
# If called with parameters it fails if something is not possible
# Flow:
# 1. Set fail - if we are in fail mode or best effort.
# 2. Sort out client_id, rp_client_sp, resources we care for RBAC.
# 3. If we are in refresh_cluster_credentials mode - attempt to reuse/recreate
# cluster service principal application and acquire client_id, client_secret
# 4. Reuse/Recreate service principal.
# 5. Sort out required rbac
def cluster_application_update(cli_ctx,
                               oc,
                               client_id,
                               client_secret,
                               refresh_cluster_credentials):
    # QUESTION: is there possible unification with the create path?

    rp_client_sp_id = None
    client_sp_id = None
    random_id = generate_random_id()

    # if any of these are set - we expect users to have access to fix rbac so we fail
    # common for 1 and 2 flows
    fail = client_id or client_secret or refresh_cluster_credentials

    aad = AADManager(cli_ctx)

    # check if we can see if RP service principal exists
    try:
        rp_client_sp_id = aad.get_service_principal_id(resolve_rp_client_id())
        if not rp_client_sp_id:
            raise ResourceNotFoundError("RP service principal not found.")
    except GraphError as e:
        if fail:
            logger.error(e.message)
            raise
        logger.info(e.message)

    # refresh_cluster_credentials refreshes cluster SP application.
    # At firsts it tries to re-use existing application and generate new password.
    # If application does not exist - creates new one
    if refresh_cluster_credentials:
        try:
            app = aad.get_application_object_id_by_client_id(client_id or oc.service_principal_profile.client_id)
            if not app:
                # we were not able to find and applications, create new one
                parts = parse_resource_id(oc.cluster_profile.resource_group_id)
                cluster_resource_group = parts['resource_group']

                client_id, client_secret = aad.create_application(cluster_resource_group or 'aro-' + random_id)
            else:
                client_secret = aad.add_password(app)
        except GraphError as e:
            logger.error(e.message)
            raise

    # attempt to get/create SP if one was not found.
    try:
        client_sp_id = aad.get_service_principal_id(client_id or oc.service_principal_profile.client_id)
    except GraphError as e:
        if fail:
            logger.error(e.message)
            raise
        logger.info(e.message)

    if fail and not client_sp_id:
        client_sp_id = aad.create_service_principal(client_id or oc.service_principal_profile.client_id)

    sp_obj_ids = [sp for sp in [rp_client_sp_id, client_sp_id] if sp]
    ensure_resource_permissions(cli_ctx, oc, fail, sp_obj_ids)

    return client_id, client_secret


def resolve_rp_client_id():
    return FP_CLIENT_ID


def ensure_resource_permissions(cli_ctx, oc, fail, sp_obj_ids):
    try:
        # Get cluster resources we need to assign permissions on, sort to ensure the same order of operations
        resources = {ROLE_NETWORK_CONTRIBUTOR: sorted(get_cluster_network_resources(cli_ctx, oc)),
                     ROLE_READER: sorted(get_disk_encryption_resources(oc))}
    except (CloudError, HttpOperationError) as e:
        if fail:
            logger.error(e.message)
            raise
        logger.info(e.message)
        return

    for sp_id in sp_obj_ids:
        for role in sorted(resources):
            for resource in resources[role]:
                # Create the role assignment if it doesn't exist
                # Assume that the role assignment exists if we fail to look it up
                resource_contributor_exists = True
                try:
                    resource_contributor_exists = has_role_assignment_on_resource(cli_ctx, resource, sp_id, role)
                except CloudError as e:
                    if fail:
                        logger.error(e.message)
                        raise
                    logger.info(e.message)
                if not resource_contributor_exists:
                    assign_role_to_resource(cli_ctx, resource, sp_id, role)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from datetime import datetime
from azure.cli.core.profiles import ResourceType

from ..util import get_datetime_from_string


def exists(cmd, client, timeout=None):
    from azure.core.exceptions import HttpResponseError
    try:
        client.get_file_system_properties(timeout=timeout)
        return True
    except HttpResponseError as ex:
        from azure.cli.command_modules.storage.track2_util import _dont_fail_on_exist
        StorageErrorCode = cmd.get_models("_shared.models#StorageErrorCode",
                                          resource_type=ResourceType.DATA_STORAGE_FILEDATALAKE)
        _dont_fail_on_exist(ex, StorageErrorCode.container_not_found)
        return False


def generate_sas_fs_uri(client, cmd, file_system, permission=None,
                        expiry=None, start=None, id=None, ip=None,  # pylint: disable=redefined-builtin
                        protocol=None, cache_control=None, content_disposition=None,
                        content_encoding=None, content_language=None,
                        content_type=None, full_uri=False, as_user=False):
    generate_file_system_sas = cmd.get_models('_shared_access_signature#generate_file_system_sas')

    sas_kwargs = {}
    if as_user:
        user_delegation_key = client.get_user_delegation_key(
            get_datetime_from_string(start) if start else datetime.utcnow(),
            get_datetime_from_string(expiry))

    sas_token = generate_file_system_sas(account_name=client.account_name, file_system_name=file_system,
                                         credential=user_delegation_key if as_user else client.credential.account_key,
                                         permission=permission, expiry=expiry, start=start, policy_id=id,
                                         ip=ip, protocol=protocol,
                                         cache_control=cache_control, content_disposition=content_disposition,
                                         content_encoding=content_encoding, content_language=content_language,
                                         content_type=content_type, **sas_kwargs)

    if full_uri:
        t_file_system_client = cmd.get_models('_file_system_client#FileSystemClient')
        file_system_client = t_file_system_client(account_url=client.url, file_system_name=file_system,
                                                  credential=sas_token)
        return file_system_client.url

    return sas_token


def list_deleted_path(client, marker=None, num_results=None, path_prefix=None, timeout=None, **kwargs):
    from ..track2_util import list_generator

    generator = client.list_deleted_paths(path_prefix=path_prefix, timeout=timeout,
                                          results_per_page=num_results, **kwargs)

    pages = generator.by_page(continuation_token=marker)  # BlobPropertiesPaged
    result = list_generator(pages=pages, num_results=num_results)

    if pages.continuation_token:
        next_marker = {"nextMarker": pages.continuation_token}
        result.append(next_marker)

    return result


def set_service_properties(cmd, client, delete_retention=None, delete_retention_period=None,
                           enable_static_website=False, index_document=None, error_document_404_path=None):
    parameters = client.get_service_properties()
    # update
    kwargs = {}
    delete_retention_policy = cmd.get_models('_models#RetentionPolicy',
                                             resource_type=ResourceType.DATA_STORAGE_FILEDATALAKE)()
    if parameters.get('delete_retention_policy', None):
        delete_retention_policy = parameters['delete_retention_policy']
    if delete_retention is not None:
        delete_retention_policy.enabled = delete_retention
    if delete_retention_period is not None:
        delete_retention_policy.days = delete_retention_period
    delete_retention_policy.allow_permanent_delete = False

    static_website = cmd.get_models('_models#StaticWebsite',
                                    resource_type=ResourceType.DATA_STORAGE_FILEDATALAKE)()
    if parameters.get('static_website', None):
        static_website = parameters['static_website']

    if static_website is not None:
        static_website.enabled = enable_static_website
    if index_document is not None:
        static_website.index_document = index_document
    if error_document_404_path is not None:
        static_website.error_document_404_path = error_document_404_path

    if parameters.get('hour_metrics', None):
        kwargs['hour_metrics'] = parameters['hour_metrics']
    if parameters.get('logging', None):
        kwargs['logging'] = parameters['logging']
    if parameters.get('minute_metrics', None):
        kwargs['minute_metrics'] = parameters['minute_metrics']
    if parameters.get('cors', None):
        kwargs['cors'] = parameters['cors']

    # checks
    if delete_retention_policy and delete_retention_policy.enabled and not delete_retention_policy.days:
        from azure.cli.core.azclierror import InvalidArgumentValueError
        raise InvalidArgumentValueError("must specify days-retained")

    client.set_service_properties(delete_retention_policy=delete_retention_policy, static_website=static_website,
                                  **kwargs)
    return client.get_service_properties()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core import AzCommandsLoader
from azure.cli.core.commands import AzArgumentContext, CliCommandType

from azure.cli.command_modules.monitor._help import helps  # pylint: disable=unused-import


# pylint: disable=line-too-long
class MonitorArgumentContext(AzArgumentContext):

    def resource_parameter(self, dest, arg_group=None, required=True, skip_validator=False, alias='resource',
                           preserve_resource_group_parameter=False):
        from azure.cli.command_modules.monitor.validators import get_target_resource_validator
        self.argument(dest, options_list='--{}'.format(alias), arg_group=arg_group, required=required,
                      validator=get_target_resource_validator(
                          dest, required, alias=alias,
                          preserve_resource_group_parameter=preserve_resource_group_parameter) if not skip_validator else None,
                      help="Name or ID of the target resource.")
        self.extra('namespace', options_list='--{}-namespace'.format(alias), arg_group=arg_group,
                   help="Target resource provider namespace.")
        self.extra('parent', options_list='--{}-parent'.format(alias), arg_group=arg_group,
                   help="Target resource parent path, if applicable.")
        self.extra('resource_type', options_list='--{}-type'.format(alias), arg_group=arg_group,
                   help="Target resource type. Can also accept namespace/type format (Ex: 'Microsoft.Compute/virtualMachines')")
        self.extra('resource_group_name', options_list=['--resource-group', '-g'], arg_group=arg_group)


class MonitorCommandsLoader(AzCommandsLoader):

    def __init__(self, cli_ctx=None):
        from azure.cli.core.profiles import ResourceType
        monitor_custom = CliCommandType(
            operations_tmpl='azure.cli.command_modules.monitor.custom#{}')
        super(MonitorCommandsLoader, self).__init__(cli_ctx=cli_ctx,
                                                    resource_type=ResourceType.MGMT_MONITOR,
                                                    argument_context_cls=MonitorArgumentContext,
                                                    custom_command_type=monitor_custom)

    def load_command_table(self, args):
        from azure.cli.command_modules.monitor.commands import load_command_table
        from azure.cli.core.aaz import load_aaz_command_table
        try:
            from . import aaz
        except ImportError:
            aaz = None
        if aaz:
            load_aaz_command_table(
                loader=self,
                aaz_pkg_name=aaz.__name__,
                args=args
            )
        load_command_table(self, args)
        return self.command_table

    def load_arguments(self, command):
        from azure.cli.command_modules.monitor._params import load_arguments
        load_arguments(self, command)


COMMAND_LOADER_CLS = MonitorCommandsLoader
